/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : Parsery.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.org
 * Creation date: 15.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Parsery.h"
//#include <QtDebug>


//###################################################################
//#                                                                 #
//#                          Parser                                 #
//#                                                                 #
//###################################################################

const QString CBtParser::Path        = "path";
const QString CBtParser::Gravity     = "gravity";
const QString CBtParser::Transparent = "transparent";
const QString CBtParser::Margin      = "margin";
const QString CBtParser::Scale       = "scale";


//*******************************************************************
// parse_color                                             PROTECTED
//*******************************************************************
QColor CBtParser::parse_color( const QString& text ) const
{
   const QStringList token = text.split( ',', QString::SkipEmptyParts );
   if( token.size() == 3 ) {
      const int r = token[ 0 ].toInt();
      const int g = token[ 1 ].toInt();
      const int b = token[ 2 ].toInt();
      if(( r >= 0 ) && ( r <= 255 )) {
         if(( g >= 0 ) && (g <= 255 )) {
            if(( b >= 0 ) && ( b <= 255 )) {
               return QColor( r, g, b );
            }
         }
      }
   }
   return QColor();
}
// end of parse_color

//*******************************************************************
// im_color                                                   PUBLIC
//*******************************************************************
Magick::Color CBtParser::im_color( const QColor& color, const int transparency ) const
{
   const double r = static_cast<double>( color.red  () * MaxRGB ) / 255.0;
   const double g = static_cast<double>( color.green() * MaxRGB ) / 255.0;
   const double b = static_cast<double>( color.blue () * MaxRGB ) / 255.0;
   const double a = transparency * MaxRGB * 0.01;
   
   return Magick::Color( r, g, b, a );
}
// end of im_color


//###################################################################
//#                                                                 #
//#                       WatermarkParser                           #
//#                                                                 #
//###################################################################

const QString CBtWatermarkParser::NorthWestGravity = "NorthWestGravity";
const QString CBtWatermarkParser::NorthGravity     = "NorthGravity";
const QString CBtWatermarkParser::NorthEastGravity = "NorthEastGravity";
const QString CBtWatermarkParser::WestGravity      = "WestGravity";
const QString CBtWatermarkParser::CenterGravity    = "CenterGravity";
const QString CBtWatermarkParser::EastGravity      = "EastGravity";
const QString CBtWatermarkParser::SouthWestGravity = "SouthWestGravity";
const QString CBtWatermarkParser::SouthGravity     = "SouthGravity";
const QString CBtWatermarkParser::SouthEastGravity = "SouthEastGravity";

//*******************************************************************
// gravity                                                    PUBLIC
//*******************************************************************
void CBtWatermarkParser::gravity( const QString& text )
{
   if( text == NorthWestGravity ) {
      gravity_ = Magick::NorthWestGravity;
      return;
   }
   if( text == NorthGravity ) {
      gravity_ = Magick::NorthGravity;
      return;
   }
   if( text == NorthEastGravity ) {
      gravity_ = Magick::NorthEastGravity;
      return;
   }
   if( text == WestGravity ) {
      gravity_ = Magick::WestGravity;
      return;
   }
   if( text == CenterGravity ) {
      gravity_ = Magick::CenterGravity;
      return;
   }
   if( text == EastGravity ) {
      gravity_ = Magick::EastGravity;
      return;
   }
   if( text == SouthWestGravity ) {
      gravity_ = Magick::SouthWestGravity;
      return;
   }
   if( text == SouthGravity ) {
      gravity_ = Magick::SouthGravity;
      return;
   }
   if( text == SouthEastGravity ) {
      gravity_ = Magick::SouthEastGravity;
      return;
   }

   gravity_ = Magick::ForgetGravity;
}
// end of gravity

//*******************************************************************
// gravity2string                                            PRIVATE
//*******************************************************************
QString CBtWatermarkParser::gravity2string() const
{
   switch( gravity_ ) {
      case Magick::NorthWestGravity:
         return NorthWestGravity;
      case Magick::NorthGravity:
         return NorthGravity;
      case Magick::NorthEastGravity:
         return NorthEastGravity;
      case Magick::WestGravity:
         return WestGravity;
      case Magick::CenterGravity:
         return CenterGravity;
      case Magick::EastGravity:
         return EastGravity;
      case Magick::SouthWestGravity:
         return SouthWestGravity;
      case Magick::SouthGravity:
         return SouthGravity;
      case Magick::SouthEastGravity:
         return SouthEastGravity;
      default:
         return QString();
   }
}
// end of gravity2string

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtWatermarkParser::command() const
{
   QString text = QString();
   if( !valid() ) return text;

   text += Path;
   text += '=';
   text += path_;
   text += ';';
   text += Gravity;
   text += '=';
   text += gravity2string();
   text += ';';
   text += Transparent;
   text += '=';
   text += QString::number( transparency_ );

   if(( x_margin_ != -1 ) && ( y_margin_ != -1 )) {
      text += ';';
      text += Margin;
      text += '=';
      text += QString::number( x_margin_ );
      text += ',';
      text += QString::number( y_margin_ );
   }
   if( scale_ != -1 ) {
      text += ';';
      text += Scale;
      text += '=';
      text += QString::number( scale_ );
   }

   return text;
}
// end of command

//*******************************************************************
// parse                                                     PRIVATE
//*******************************************************************
void CBtWatermarkParser::parse( const QString& text )
{
   const QStringList token = text.split( ';', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( '=' );
      if( idx != -1 ) {
         const QString cmd = token[i].left( idx );
         const QString par = token[i].mid( idx + 1 );
         if( Path == cmd ) {
            path_ = par;
         }
         else if( Gravity == cmd ) {
            gravity( par );
         }
         else if( Transparent == cmd ) {
            transparency_ = par.toInt();
         }
         else if( Margin == cmd ) {
            x_margin_ = par.section( ',', 0, 0 ).toInt();
            y_margin_ = par.section( ',', 1, 1 ).toInt();

         }
         else if( Scale == cmd ) {
            scale_ = par.toInt();
         }
      }
   }
}
// end of parse


//###################################################################
//#                                                                 #
//#                         ResizeParser                            #
//#                                                                 #
//###################################################################

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtResizeParser::command() const
{
   static const QString Value   = "value=%1,%2";
   static const QString Percent = "percent=%1";
   static const QString Scale   = "scale";

   QString text = QString();
   if( !valid() ) return text;
   
   if( newsize() ) {
      text += Value.arg( width_ ).arg( height_ );
   }
   else {
      text += Percent.arg( percent_ );
   }
   if( scale_ ) {
      text += ';';
      text += Scale;
   }
   return text;
}
// end of command

//*******************************************************************
// parse                                                     PRIVATE
//*******************************************************************
void CBtResizeParser::parse( const QString& text )
{
   static const QString Value   = "value";
   static const QString Percent = "percent";
   static const QString Scale   = "scale";

   const QStringList token = text.split( ';', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( '=' );
      if( idx != -1 ) {
         const QString cmd = token[i].left( idx );
         const QString par = token[i].mid( idx + 1 );
         if( Value == cmd ) {
            width_  = par.section( ',', 0, 0 ).toInt();
            height_ = par.section( ',', 1, 1 ).toInt();
         }
         else if( Percent == cmd ) {
            percent_ = par.toInt();
         }
      }
      else if( token[i] == Scale ) {
         scale_ = true;
      }
   }
}
// end of parse


//###################################################################
//#                                                                 #
//#                        AnnotateParser                           #
//#                                                                 #
//###################################################################

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtAnnotateParser::command() const
{
   QString text = QString();
   if( !valid() ) return text;
   
   static const QString Picture     = "picture=%1";
   static const QString Text        = "text=%1";
   static const QString Color       = "color=%1,%2,%3";
   static const QString Transparent = "transparent=%1";
   static const QString Margin      = "margin=%1,%2";

   if( !fpath_.isEmpty() ) {
      text += Picture.arg( fpath_ );
   }
   else if( !text_.isEmpty() ) {
      text += Text.arg( text_ );
   }
   else {
      return text;
   }

   text += ';';
   text += Color.arg( color_.red() ).arg( color_.green() ).arg( color_.blue() );
   text += ';';
   text += Transparent.arg( transparency_ );
   text += ';';
   text += Margin.arg( x_margin_ ).arg( y_margin_ );
   
   return text;
}
// end of command

//*******************************************************************
// parse                                                      PUBLIC
//*******************************************************************
void CBtAnnotateParser::parse( const QString& text )
{
   static const QString Picture     = "picture";
   static const QString Text        = "text";
   static const QString Color       = "color";
   static const QString Transparent = "transparent";
   static const QString Margin      = "margin";

   const QStringList token = text.split( ';', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( '=' );
      if( idx != -1 ) {
         const QString cmd = token[i].left( idx );
         const QString par = token[i].mid( idx + 1 );
         if( Picture == cmd ) {
            fpath_ = par;
         }
         else if( Text == cmd ) {
            text_ = par;
         }
         else if( Color == cmd ) {
            color_ = parse_color( par );
         }
         else if( Transparent == cmd ) {
            transparency_ = par.toInt();
         }
         else if( Margin == cmd ) {
            x_margin_ = par.section( ',', 0, 0 ).toInt();
            y_margin_ = par.section( ',', 1, 1 ).toInt();
         }
      }
   }
}
// end of parse


//###################################################################
//#                                                                 #
//#                        RotateParser                             #
//#                                                                 #
//###################################################################

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtRotateParser::command() const
{
   static const QString Angle = "angle=%1";
   static const QString Color = "color=%1,%2,%3";
   static const QString Trans = "transparency=%1";

   QString text = QString();
   if( !valid() ) return text;

   text += Angle.arg( angle_ );
   text += ';';
   text += Color.arg( color_.red() ).arg( color_.green() ).arg( color_.blue() );
   text += ';';
   text += Trans.arg( transparency_ );

   return text;   
}
// end of command;

//*******************************************************************
// parse                                                      PUBLIC
//*******************************************************************
void CBtRotateParser::parse( const QString& text )
{
   static const QString Angle = "angle";
   static const QString Color = "color";
   static const QString Trans = "transparency";

   const QStringList token = text.split( ';', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( '=' );
      if( idx != -1 ) {
         const QString cmd = token[i].left( idx );
         const QString par = token[i].mid( idx + 1 );
         if( Angle == cmd ) {
            angle_ = par.toInt();
         }
         else if( Color == cmd ) {
            color_ = parse_color( par );
         }
         else if( Trans == cmd ) {
            transparency_ = par.toInt();
         }
      }
   }
}
// end of parse

//*******************************************************************
// im_color                                                   PUBLIC
//*******************************************************************
Magick::Color CBtRotateParser::im_color() const
{
   return CBtParser::im_color( color_, transparency_ );
}
// end of im_color

//###################################################################
//#                                                                 #
//#                        BorderInfo                               #
//#                                                                 #
//###################################################################

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtBorderInfo::command() const
{
   static const QString Color = "color=%1,%2,%3";
   static const QString Width = "width=%1";
   static const QString Trans = "transparency=%1";
   static const QString Order = "order=%1";

   QString text = QString();
   if( !valid() ) return text;
   
   text += Color.arg( color_.red() ).arg( color_.green() ).arg( color_.blue() );
   text += ';';
   text += Width.arg( width_ );
   text += ';';
   text += Trans.arg( transparency_ );
   text += ';';
   text += Order.arg( order_ );

   return text;
}
// end of command

//*******************************************************************
// parse                                                      PUBLIC
//*******************************************************************
void CBtBorderInfo::parse( const QString& text )
{
   static const QString Color = "color";
   static const QString Width = "width";
   static const QString Trans = "transparency";
   static const QString Order = "order";
   
   const QStringList token = text.split( ';', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( '=' );
      if( idx != -1 ) {
         const QString ident = token[i].left( idx );
         const QString value = token[i].mid( idx + 1 );
         if( Color == ident ) {
            color_ = parse_color( value );
         }
         else if( Width == ident ) {
            width_ = value.toInt();
         }
         else if( Trans == ident ) {
            transparency_ = value.toInt();
         }
         else if( Order == ident ) {
            order_ = value.toInt();
         }
      }
   }
}
// end of parse

//*******************************************************************
// im_color                                                   PUBLIC
//*******************************************************************
Magick::Color CBtBorderInfo::im_color() const
{
   return CBtParser::im_color( color_, transparency_ );
}
// end of im_color

//###################################################################
//#                                                                 #
//#                        BorderParser                             #
//#                                                                 #
//###################################################################

//*******************************************************************
// command                                                    PUBLIC
//*******************************************************************
QString CBtBorderParser::command() const
{
   QString text = QString();

   if( top().valid() ) {
      text += "top:";
      text += top().command();
   }
   if( bottom().valid() ) {
      if( !text.isEmpty() ) text += '#';
      text += "bottom:";
      text += bottom().command();
   }
   if( left().valid() ) {
      if( !text.isEmpty() ) text += '#';
      text += "left:";
      text += left().command();
   }
   if( right().valid() ) {
      if( !text.isEmpty() ) text += '#';
      text += "right:";
      text += right().command();
   }
   
   return text;
}
// end of command

//*******************************************************************
// parse                                                      PUBLIC
//*******************************************************************
void CBtBorderParser::parse( const QString& text )
{
   const QStringList token = text.split( '#', QString::SkipEmptyParts );
   const int n = token.size();
   for( int i = 0; i < n; ++i ) {
      const int idx = token[i].indexOf( ':' );
      if( idx != -1 ) {
         const QString cmd = token[i].left( idx );
         const QString par = token[i].mid( idx + 1 );

              if( "top"    == cmd ) top   ().parse( par );
         else if( "bottom" == cmd ) bottom().parse( par );
         else if( "left"   == cmd ) left  ().parse( par );
         else if( "right"  == cmd ) right ().parse( par );
      }
   }
}
// end of parser
