#ifndef INCLUDED_Parsery_h
#define INCLUDED_Parsery_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : Parsery.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.org
 * Creation date: 15.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QString>
#include <QColor>
#include <Magick++.h>

/*------- class CBtParser:
-------------------------------------------------------------------*/
class CBtParser
{
//------- DESTRUCTION/DESTRUCTION -------
public:
   CBtParser() {}
   virtual ~CBtParser() {}

//------- CONSTANTS -------
protected:
   static const QString Path;
   static const QString Gravity;
   static const QString Transparent;
   static const QString Margin;
   static const QString Scale;

//------- METHODS -------
protected:
   QColor        parse_color( const QString&      ) const;
   Magick::Color im_color   ( const QColor&, int  ) const;
};

/*------- class CBtWatermarkParser:
-------------------------------------------------------------------*/
class CBtWatermarkParser : public CBtParser
{
//------- CONSTRUCTION/DESTRUCTION -------
public:
   CBtWatermarkParser()
   : CBtParser     ()
   , path_         ( QString() )
   , gravity_      ( Magick::ForgetGravity )
   , transparency_ ( -1 )
   , x_margin_     ( -1 )
   , y_margin_     ( -1 )
   , scale_        ( -1 )
   {}
   CBtWatermarkParser( const QString& text )
   : CBtParser     ()
   , path_         ( QString() )
   , gravity_      ( Magick::ForgetGravity )
   , transparency_ ( -1 )
   , x_margin_     ( -1 )
   , y_margin_     ( -1 )
   , scale_        ( -1 )
   {
      parse( text );
   }

//------- CONSTANTS -------
private:
   static const QString NorthWestGravity;
   static const QString NorthGravity;
   static const QString NorthEastGravity;
   static const QString WestGravity;
   static const QString CenterGravity;
   static const QString EastGravity;
   static const QString SouthWestGravity;
   static const QString SouthGravity;
   static const QString SouthEastGravity;

//------- MEMBERS -------
private:
   QString             path_;
   Magick::GravityType gravity_;
   int                 transparency_;
   int                 x_margin_;
   int                 y_margin_;
   int                 scale_;

//------- METHODS -------
public:
   const QString&      path        () const { return path_;         }
   Magick::GravityType gravity     () const { return gravity_;      }
   int                 transparency() const { return transparency_; }
   int                 x_margin    () const { return x_margin_;     }
   int                 y_margin    () const { return y_margin_;     }
   int                 scale       () const { return scale_;        }
   bool                valid       () const {
      if( path_.isEmpty() ) return false;
      if( Magick::ForgetGravity == gravity_ ) return false;
      if( -1 == transparency_ ) return false;
      return true;
   }

   void path        ( const QString& text ) { path_ = text; }
   void gravity     ( const QString&      );
   void gravity     ( const Magick::GravityType value ) { gravity_ = value; }
   void transparency( const int value     ) { transparency_ = value; }
   void x_margin    ( const int value     ) { x_margin_     = value; }
   void y_margin    ( const int value     ) { y_margin_     = value; }
   void scale       ( const int value     ) { scale_        = value; }

   QString command() const;
private:
   QString gravity2string() const;
   void parse( const QString& );
};

/*------- class CBtResizeParser:
-------------------------------------------------------------------*/
class CBtResizeParser : public CBtParser
{
//------- CONSTRUCTION/DESTRUCTION -------
public:
   CBtResizeParser()
   : CBtParser()
   , width_   ( -1 )
   , height_  ( -1 )
   , percent_ ( -1 )
   , scale_   ( false )
   {}
   CBtResizeParser( const QString& text )
   : CBtParser()
   , width_   ( -1 )
   , height_  ( -1 )
   , percent_ ( -1 )
   , scale_   ( false )
   {
      parse( text );
   } 
   ~CBtResizeParser()
   {}
   CBtResizeParser( const CBtResizeParser& rhs )
   : CBtParser()
   , width_   ( rhs.width_   )
   , height_  ( rhs.height_  )
   , percent_ ( rhs.percent_ )
   , scale_   ( rhs.scale_   )
   {}
   CBtResizeParser& operator=( const CBtResizeParser& rhs ) {
      if( this != &rhs ) {
         width_   = rhs.width_;
         height_  = rhs.height_;
         percent_ = rhs.percent_;
         scale_   = rhs.scale_;
      }
      return *this;
   }
//------- MEMBERS -------
private:
   int  width_;
   int  height_;
   int  percent_;
   bool scale_;

//------- METHODS -------
public:
   int  width    () const { return width_;   }
   int  height   () const { return height_;  }
   int  percent  () const { return percent_; }
   bool scale    () const { return scale_;   }
   bool newsize  () const { return (( width_ != -1 ) && ( height_ != -1 )); }

   void width     ( const int  value ) { width_   = value; }
   void height    ( const int  value ) { height_  = value; }
   void percent   ( const int  value ) { percent_ = value; }
   void use_scale ()                   { scale_   = true;  }

   bool valid() const {
      return ((( width_ != -1 ) && ( height_ != -1 )) || ( percent_ != -1 ));
   }
   
   QString command() const;
   void parse( const QString& );
};

/*------- class CBtAnnotateParser:
-------------------------------------------------------------------*/
class CBtAnnotateParser : public CBtParser
{
//------- CONSTRUCTION/DESTRUCTION -------
public:
   CBtAnnotateParser()
   : CBtParser     ()
   , fpath_        ( QString() )
   , text_         ( QString() )
   , color_        ( QColor () )
   , transparency_ ( -1        )
   , x_margin_     ( -1        )
   , y_margin_     ( -1        )
   {}
   CBtAnnotateParser( const QString& text )
   : CBtParser     ()
   , fpath_        ( QString() )
   , text_         ( QString() )
   , color_        ( QColor () )
   , transparency_ ( -1        )
   , x_margin_     ( -1        )
   , y_margin_     ( -1        )
   {
      parse( text );
   }
   ~CBtAnnotateParser()
   {}
   CBtAnnotateParser( const CBtAnnotateParser& rhs )
   : CBtParser     ()
   , fpath_        ( rhs.fpath_        )
   , text_         ( rhs.text_         )
   , color_        ( rhs.color_        )
   , transparency_ ( rhs.transparency_ )
   , x_margin_     ( rhs.x_margin_     )
   , y_margin_     ( rhs.y_margin_     )
   {}
   CBtAnnotateParser& operator=( CBtAnnotateParser& rhs )
   {
      if( this != &rhs ) {
         fpath_        = rhs.fpath_;
         text_         = rhs.text_;
         color_        = rhs.color_;
         transparency_ = rhs.transparency_;
         x_margin_     = rhs.x_margin_;
         y_margin_     = rhs.y_margin_;
      }
      return *this;
   }

//------- MEMBERS -------
private:
   QString fpath_;
   QString text_;
   QColor  color_;
   int     transparency_;
   int     x_margin_;
   int     y_margin_;
   
//------- METHODS -------
public:
   void fpath        ( const QString& value ) { fpath_        = value; }
   void text         ( const QString& value ) { text_         = value; }
   void color        ( const QColor&  value ) { color_        = value; }
   void transparency ( const int      value ) { transparency_ = value; }
   void x_margin     ( const int      value ) { x_margin_     = value; }
   void y_margin     ( const int      value ) { y_margin_     = value; }

   const QString& fpath       () const { return fpath_;        }
   const QString& text        () const { return text_;         }
   const QColor&  color       () const { return color_;        }
   int            transparency() const { return transparency_; }
   int            x_margin    () const { return x_margin_;     }
   int            y_margin    () const { return y_margin_;     }

   Magick::Color im_color() const {
      return CBtParser::im_color( color_, transparency_ );
   }
   bool valid() const {
      if( fpath_.isEmpty() && text_.isEmpty()) return false;
      if( -1 == transparency_ ) return false;
      if( -1 == x_margin_ ) return false;
      if( -1 == y_margin_ ) return false;
      return color_.isValid();
   }

   QString command() const;
   void parse( const QString& );
};

/*------- class CBtRotateParser:
-------------------------------------------------------------------*/
class CBtRotateParser : public CBtParser
{
//------- CONSTRUCTION/DESTRUCTION -------
public:
   CBtRotateParser()
   : CBtParser    ()
   , angle_       ( -1       )
   , color_       ( QColor() )
   , transparency_( -1       )
   {}
   CBtRotateParser( const QString& text )
   : CBtParser    ()
   , angle_       ( -1       )
   , color_       ( QColor() )
   , transparency_( -1       )
   {
      parse( text );
   }
   CBtRotateParser( const CBtRotateParser& rhs )
   : angle_       ( rhs.angle_        )
   , color_       ( rhs.color_        )
   , transparency_( rhs.transparency_ )
   {}
   CBtRotateParser& operator=( const CBtRotateParser& rhs ) {
      if( this != &rhs ) {
         angle_       = rhs.angle_;
         color_       = rhs.color_;
         transparency_= rhs.transparency_ ;
      }
      return *this;
   }
   
//------- MEMBERS -------
private:
   int    angle_;
   QColor color_;
   int    transparency_;

//------- METHODS -------
public:
   int           angle       () const { return angle_;        }
   const QColor& color       () const { return color_;        }
   int           transparency() const { return transparency_; }
   Magick::Color im_color    () const;

   void angle       ( const int     value ) { angle_        = value; }
   void color       ( const QColor& color ) { color_        = color; }
   void transparency( const int     value ) { transparency_ = value; }

   QString command() const;
   void    parse  ( const QString& );

   bool valid() const {
      if( -1 == angle_ ) return false;
      if( -1 == transparency_ ) return false;
      return true;
   }
};

/*------- class CBtBorderInfo & CBtBorderParser:
-------------------------------------------------------------------*/
class CBtBorderInfo : public CBtParser
{
//------- CONSTRUCTION / DESTRUCTION -------
public:
   CBtBorderInfo()
   : CBtParser     ()
   , color_        ( QColor( 0, 0, 0 ) )
   , transparency_ ( -1 )
   , width_        ( -1 )
   , order_        ( -1 )
   {}
   CBtBorderInfo( const CBtBorderInfo& rhs )
   : CBtParser     ()
   , color_        ( rhs.color_ )
   , transparency_ ( rhs.transparency_ )
   , width_        ( rhs.width_ )
   , order_        ( rhs.order_ )
   {}
   CBtBorderInfo& operator=( const CBtBorderInfo& rhs ) {
      if( this != &rhs ) {
         color_        = rhs.color_;
         transparency_ = rhs.transparency_;
         width_        = rhs.width_;
         order_        = rhs.order_;
      }
      return *this;
   }
//------- MEMBERS -------
private:
   QColor color_;
   int    transparency_;
   int    width_;
   int    order_;
   
//------- METHODS -------
public:
   bool valid() const {
      if( -1 == transparency_ ) return false;
      if( -1 == width_        ) return false;
      if( -1 == order_        ) return false;
      return true;
   }
   bool acceptable() const {
      if( !valid()             ) return false;
      if( transparency_  <   0 ) return false;
      if( transparency_  > 100 ) return false;
      if( width_        <=   0 ) return false;
      if( order_         <   1 ) return false;
      if( order_         >   4 ) return false;
      return true;
   }
   const QColor& qt_color    () const { return color_; }
   Magick::Color im_color    () const;
   int           transparency() const { return transparency_; }
   int           width       () const { return width_; }
   int           order       () const { return order_; }

   void color       ( const QColor& color ) { color_ = color; }
   void transparency( const int     value ) { transparency_ = value; }
   void width       ( const int     value ) { width_ = value; }
   void order       ( const int     value ) { order_ = value; }

   QString command() const;
   void    parse  ( const QString& );
};


class CBtBorderParser
{
public:
   CBtBorderParser()
   {}
   CBtBorderParser( const QString& text ) {
      parse( text );
   }
private:
   CBtBorderParser( const CBtBorderParser& );
   CBtBorderParser& operator=( const CBtBorderParser& );

//------- MEMBERS -------
private:
   CBtBorderInfo top_;
   CBtBorderInfo bottom_;
   CBtBorderInfo left_;
   CBtBorderInfo right_;

//------- METHODS -------
public:
   void top   ( const CBtBorderInfo& info ) { top_    = info; }
   void bottom( const CBtBorderInfo& info ) { bottom_ = info; }
   void left  ( const CBtBorderInfo& info ) { left_   = info; }
   void right ( const CBtBorderInfo& info ) { right_  = info; }

   const CBtBorderInfo& top   () const { return top_;    }
   const CBtBorderInfo& bottom() const { return bottom_; }
   const CBtBorderInfo& left  () const { return left_;   }
   const CBtBorderInfo& right () const { return right_;  }
   
   CBtBorderInfo& top   () { return top_;    }
   CBtBorderInfo& bottom() { return bottom_; }
   CBtBorderInfo& left  () { return left_;   }
   CBtBorderInfo& right () { return right_;  }

   QString command() const;
   void    parse  ( const QString& );
};

#endif // INCLUDED_Parsery_h
