/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigAnnotate.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 25.03.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Parsery.h"
#include "QBtShared.h"
#include "QBtConfigAnnotate.h"
#include "QBtColorDemo.h"
#include "QBtEventsController.h"
#include "QBtMagic.h"
#include <QVBoxLayout>
#include <QGridLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QSpinBox>
#include <QLabel>
#include <QColorDialog>
#include <QRadioButton>
#include <QFileDialog>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigAnnotate::Caption     = QT_TR_NOOP( "Annonate" );
const char* const QBtConfigAnnotate::Picture     = QT_TR_NOOP( "&Picture:" );
const char* const QBtConfigAnnotate::Text        = QT_TR_NOOP( "&Text:" );
const char* const QBtConfigAnnotate::Background  = QT_TR_NOOP( "&Background:" );
const char* const QBtConfigAnnotate::Transparent = QT_TR_NOOP( "Trans&parent:" );
const char* const QBtConfigAnnotate::XMargin     = QT_TR_NOOP( "&x-margin:" );
const char* const QBtConfigAnnotate::YMargin     = QT_TR_NOOP( "&y-margin:" );

//*******************************************************************
// QBtConfigAnnotate                                     CONSTRUCTOR
//*******************************************************************
QBtConfigAnnotate::QBtConfigAnnotate( QWidget* const parent, const QString& path )
: QBtConfigBaseDialog( parent, path )
, picture_radio_btn_ ( new QRadioButton( tr( Picture ) ) )
, picture_path_edit_ ( new QLineEdit )
, picture_path_btn_  ( new QPushButton( QIcon( ":/img/folder.png" ), QString() ) )
, text_radio_btn_    ( new QRadioButton( tr( Text ) ) )
, text_edit_         ( new QLineEdit )
, color_demo_        ( new QBtColorDemo )
, color_btn_         ( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
, trans_spin_box_    ( new QSpinBox )
, x_margin_spin_box_ ( new QSpinBox )
, y_margin_spin_box_ ( new QSpinBox )
{
   setWindowTitle( tr( Caption ) );

   picture_radio_btn_->setChecked( true );
   text_edit_->setEnabled( false );
   trans_spin_box_->setRange( 0, 100 );
   trans_spin_box_->setValue( 0 );
   trans_spin_box_->setSuffix( "%" );
   x_margin_spin_box_->setRange( 0, 10000 );
   x_margin_spin_box_->setValue( 2 );
   x_margin_spin_box_->setSuffix( "px" );
   y_margin_spin_box_->setRange( 0, 10000 );
   y_margin_spin_box_->setValue( 2 );
   y_margin_spin_box_->setSuffix( "px" );

   QLabel* color_prompt    = new QLabel( tr( Background ) );
   QLabel* trans_prompt    = new QLabel( tr( Transparent ) );
   QLabel* x_margin_prompt = new QLabel( tr( XMargin ) );
   QLabel* y_margin_prompt = new QLabel( tr( YMargin ) );
  
   color_prompt->setBuddy( color_btn_ );
   trans_prompt->setBuddy( trans_spin_box_ );
   x_margin_prompt->setBuddy( x_margin_spin_box_ );
   y_margin_prompt->setBuddy( y_margin_spin_box_ );

   QHBoxLayout* const picture_layout = new QHBoxLayout;
   picture_layout->addWidget( picture_path_edit_ );
   picture_layout->addWidget( picture_path_btn_  );
   QGridLayout* const what_layout = new QGridLayout;
   what_layout->addWidget( picture_radio_btn_, 0, 0 );
   what_layout->addLayout( picture_layout    , 0, 1 );
   what_layout->addWidget( text_radio_btn_   , 1, 0 );
   what_layout->addWidget( text_edit_        , 1, 1 );

   QGridLayout* const config_layout = new QGridLayout;
   config_layout->addWidget( color_prompt, 0, 0 );
   config_layout->addWidget( color_demo_, 0, 1 );
   config_layout->addWidget( color_btn_ , 0, 2 );
   config_layout->addWidget( trans_prompt, 1, 0 );
   config_layout->addWidget( trans_spin_box_, 1, 1 );

   QGridLayout* const margin_layout = new QGridLayout;
   margin_layout->addWidget( x_margin_prompt   , 0, 0 );
   margin_layout->addWidget( x_margin_spin_box_, 0, 1 );
   margin_layout->addWidget( y_margin_prompt   , 1, 0 );
   margin_layout->addWidget( y_margin_spin_box_, 1, 1 );

   QGridLayout* const tool_grid = new QGridLayout;
   tool_grid->addLayout( what_layout  , 0, 0 );
   tool_grid->addLayout( config_layout, 0, 2 );
   tool_grid->addLayout( margin_layout, 0, 4 );
   tool_grid->setColumnMinimumWidth( 1, 20 );
   tool_grid->setColumnMinimumWidth( 3, 20 );
   
   QHBoxLayout* const work_layout = new QHBoxLayout;
   work_layout->addLayout( tool_grid );
   work_layout->addStretch();
   user_layout_->addLayout( work_layout );

   //------- SIGNAL/SLOT ----------------------------------
   connect( try_, SIGNAL( clicked()), this, SLOT( try_clicked()));
   connect( accept_, SIGNAL( clicked()), this, SLOT( accept_clicked()));
   connect( picture_radio_btn_, SIGNAL(toggled(bool)), this, SLOT(picture_radio_toggled(bool)));
   connect( picture_path_btn_, SIGNAL(clicked()), this, SLOT(picture_path_clicked()));
   connect( text_radio_btn_, SIGNAL(toggled(bool)), this, SLOT(text_radio_toggled(bool)));
   connect( color_btn_, SIGNAL(clicked()), this, SLOT(color_clicked()));
   
   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::ParamsAnswer );
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigAnnotate

//*******************************************************************
// ~QBtConfigAnnotate                                     DESTRUCTOR
//*******************************************************************
QBtConfigAnnotate::~QBtConfigAnnotate()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigAnnotate

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigAnnotate::showEvent( QShowEvent* const e )
{
   QBtConfigBaseDialog::showEvent( e );
   QBtEventsController::instance()->send_event( QBtEvent::ParamsRequest, "annotate" );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigAnnotate::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::ParamsAnswer:
         update_values( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "annotate", get_command() );
         break;
   }
}
// end of customEvent

//*******************************************************************
// update_values                                             PRIVATE
//*******************************************************************
void QBtConfigAnnotate::update_values( const QString& params )
{
   CBtAnnotateParser parser( params );
   if( !parser.fpath().isEmpty() ) {
      picture_radio_btn_->setChecked( true );
      picture_path_edit_->setText( parser.fpath() );
   }
   if( !parser.text().isEmpty() ) {
      text_radio_btn_->setChecked( true );
      text_edit_->setText( parser.text() );  
   }
   color_demo_       ->background( parser.color()        );
   trans_spin_box_   ->setValue  ( parser.transparency() );
   x_margin_spin_box_->setValue  ( parser.x_margin()     );
   y_margin_spin_box_->setValue  ( parser.y_margin()     );
}
// end of update_values

//*******************************************************************
// color_clicked                                        PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik wcisnal przycisk wyboru koloru (dla tla).
//*******************************************************************
void QBtConfigAnnotate::color_clicked()
{
   const QColor new_color = QColorDialog::getColor( color_demo_->background(), this );
   if( new_color.isValid() ) {
      color_demo_->background( new_color );
   }
}
// end of color_clicked

//*******************************************************************
// picture_radio_toggled                                PRIVATE slot
//*******************************************************************
void QBtConfigAnnotate::picture_radio_toggled( const bool state )
{
   if( state ) {
      text_edit_->setEnabled( false );
      picture_path_edit_->setEnabled( true );
      picture_path_btn_->setEnabled( true );
   }
}
// end of picture_radio_toggled

//*******************************************************************
// text_radio_toggled                                   PRIVATE slot
//*******************************************************************
void QBtConfigAnnotate::text_radio_toggled( const bool state )
{
   if( state ) {
      text_edit_->setEnabled( true );
      picture_path_edit_->setEnabled( false );
      picture_path_btn_->setEnabled( false );
   }
}
// end of text_radio_toggled

//*******************************************************************
// picture_path_clicked                                 PRIVATE slot
//*******************************************************************
void QBtConfigAnnotate::picture_path_clicked()
{
   const QString path = QFileDialog::getOpenFileName( this );
   if( !path.isEmpty() ) {
      picture_path_edit_->setText( path );
   }
}
// end of picture_path_clicked

//*******************************************************************
// get_command                                               PRIVATE
//*******************************************************************
QString QBtConfigAnnotate::get_command() const
{
   CBtAnnotateParser parser;
   
   if( picture_radio_btn_->isChecked() ) {
      parser.fpath( picture_path_edit_->text() );
   }
   else if( text_radio_btn_->isChecked() ) {
      parser.text( text_edit_->text() );
   }
   parser.color( color_demo_->background() );
   parser.transparency( trans_spin_box_->value() );
   parser.x_margin( x_margin_spin_box_->value() );
   parser.y_margin( y_margin_spin_box_->value() );
   
   return parser.command();
}
// end of get_command

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigAnnotate::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.annotate( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();
}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigAnnotate::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "annotate",get_command() );
   accept(); 
}
// end of accept_clicked
