/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigBaseDialog.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 13.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBaseDialog.h"
#include "QBtEventsController.h"
#include "QBtViewer.h"
#include "QBtPicture.h"
#include "QBtShared.h"
#include <QPushButton>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QDesktopWidget>
#include <QApplication>
#include <QMessageBox>
//#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigBaseDialog::Try     = QT_TR_NOOP( "&Try" );
const char* const QBtConfigBaseDialog::TryAll  = QT_TR_NOOP( "T&ry all" );
const char* const QBtConfigBaseDialog::Reset   = QT_TR_NOOP( "&Reset" );
const char* const QBtConfigBaseDialog::Cancel  = QT_TR_NOOP( "&Cancel" );
const char* const QBtConfigBaseDialog::Accept  = QT_TR_NOOP( "&Accept" );
const char* const QBtConfigBaseDialog::Error   = QT_TR_NOOP( "Error" );
const char* const QBtConfigBaseDialog::Message = QT_TR_NOOP( "Error on picture processing\nSee Output tab." );

//*******************************************************************
// QBtConfigBaseDialog                                   CONSTRUCTOR
//*******************************************************************
QBtConfigBaseDialog::QBtConfigBaseDialog( QWidget* const parent, const QString& fpath )
: QDialog     ( parent )
, pictureA_   ( new QBtPicture )
, pictureB_   ( new QBtPicture )
, try_        ( new QPushButton( tr( Try ) ) )
, accept_     ( new QPushButton( tr( Accept ) ) )
, user_layout_( new QHBoxLayout )
, magic_      ( this )
, src_qimage_ ()
, dst_qimage_ ()
, try_all_    ( new QPushButton( tr( TryAll ) ) )
, reset_      ( new QPushButton( tr( Reset ) ) )
, cancel_     ( new QPushButton( tr( Cancel ) ) )
, fpath_      ( fpath )
, first_time_ ( true )
{
   QHBoxLayout* const pic_layout = new QHBoxLayout;
   pic_layout->addWidget( pictureA_ );
   pic_layout->addWidget( pictureB_ );
   
   QHBoxLayout* const btn_layout = new QHBoxLayout;
   btn_layout->addStretch( 100 );
   btn_layout->addWidget( try_ );
   btn_layout->addWidget( try_all_ );
   btn_layout->addWidget( reset_ );
   btn_layout->addStretch( 3 );
   btn_layout->addWidget( cancel_ );
   btn_layout->addWidget( accept_ );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addLayout( pic_layout );
   main_layout->addLayout( user_layout_ );
   main_layout->addLayout( btn_layout );
   setLayout( main_layout );

   connect( try_all_, SIGNAL( clicked() ), this, SLOT( try_all_clicked() ) );
   connect( reset_  , SIGNAL( clicked() ), this, SLOT( reset          () ) );
   connect( cancel_ , SIGNAL( clicked() ), this, SLOT( accept         () ) );
}
// end of QBtConfigBaseDialog

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigBaseDialog::showEvent( QShowEvent* const e )
{
   QDialog::showEvent( e );

   if( first_time_ ) {
      const int   screen_width   = qApp->desktop()->availableGeometry().width();
      const int   screen_height  = qApp->desktop()->availableGeometry().height();
      const float width_percent  = 90.0 / 100.0;
      const float height_percent = 60.0 / 100.0;    
      const int   dx = static_cast<int>( screen_width  * width_percent );
      const int   dy = static_cast<int>( screen_height * height_percent );
      const int   x  = ( screen_width  - dx + 1 ) >> 1;
      const int   y  = ( screen_height - dy + 1 ) >> 1;
      move( x, y );
      resize( dx, dy );
   }
   QBtShared::disable_updates( this );  
   if( first_time_ ) {
      reset();
      first_time_ = false;
   }
   resize_image( false );
   QBtShared::enable_updates( this );
}
// end of showEvent

//*******************************************************************
// resizeEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigBaseDialog::resizeEvent( QResizeEvent* e )
{
   QDialog::resizeEvent( e );
   resize_image();
}
// end of resizeEvent

//*******************************************************************
// resize_image                                              PRIVATE
//*******************************************************************
void QBtConfigBaseDialog::resize_image( const bool enable )
{
   QImage imgA = src_qimage_;
   QImage imgB = dst_qimage_;
   if( imgA.isNull() || imgB.isNull() ) return;

   if( enable ) QBtShared::disable_updates( this );  
   
   // rozmiary oryginalnych obrazow
   const int wApic = imgA.width();
   const int hApic = imgA.height();
   const int wBpic = imgB.width();
   const int hBpic = imgB.height();
   // apect ratios
   const double w_aspect = static_cast<double>( wBpic ) / static_cast<double>( wApic );
   const double h_aspect = static_cast<double>( hBpic ) / static_cast<double>( hApic );
   // rozmiary wyswietlanych obrazow
   const int wA = pictureA_->width() - 8;
   const int hA = pictureA_->height() - 8;
   const int wB = w_aspect * wA;
   const int hB = h_aspect * hA;

   if( ( imgA.width() > wA ) || ( imgA.height() > hA ) ) {
      imgA = imgA.scaled( QSize( wA, hA ), Qt::KeepAspectRatio );
   }
   if( ( imgB.width() > wB ) || ( imgB.height() > hB ) ) {
      imgB = imgB.scaled( QSize( wB, hB ), Qt::KeepAspectRatio );
   }

   pictureA_->display( imgA );
   pictureB_->display( imgB );
   
   if( enable ) QBtShared::enable_updates( this );  
}
// end of resize_image

//*******************************************************************
// reset                                                   PROTECTED
//*******************************************************************
void QBtConfigBaseDialog::reset()
{
   QBtShared::disable_updates( this );  
   magic_.clear();
   if( magic_.read( fpath_ ) ) {
      src_qimage_ = dst_qimage_ = magic_.qimage();
      pictureA_->display( src_qimage_ );
      pictureB_->display( dst_qimage_ );
   }
   resize_image( false );
   QBtShared::enable_updates( this );  
}
// end of reset

//*******************************************************************
// try_all_checked                                      PRIVATE slot
//*******************************************************************
void QBtConfigBaseDialog::try_all_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::CommandsAndParamsRequest );
}
// end of try_all_clicked

//*******************************************************************
// try_all                                                 PROTECTED
//*******************************************************************
void QBtConfigBaseDialog::try_all( const QStringList& data, const QString& cmd, const QString& pars )
{
   if( pars.isEmpty() ) return;
   
   Strings command = Strings();
   Strings params  = Strings();
   {
      bool used = false;
      const int n = data.size();
      for( int i = 0; i < n; ++i ) {
         const QString tmp1 = data[i].section( '#', 0, 0 );
         const QString tmp2 = data[i].section( '#', 1 );
         if( !used && ( tmp1 == cmd ) ) {
            command.push_back( cmd );
            params.push_back( pars );
            used = true;
         }
         else {
            command.push_back( tmp1 );
            params.push_back( tmp2 );
         }
      }
      if( !used ) {
         command.push_back( cmd );
         params.push_back( pars );
      }
   }

   bool error = false;
   QBtShared::disable_updates( this );
   QBtMagic tmp = magic_;
   try {
      const int n = command.size();
      for( int i = 0; i < n; ++i ) {
         tmp.process( command[i], params[i] );
      }
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) {
      QMessageBox::critical( this, tr( Error ), tr( Message ) );
   }
   else {
      QBtViewer dialog( this, tmp.qimage() );
      dialog.exec();
   }
}
// end of try_all
