/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigRotate.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.org
 * Creation date: 13.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigRotate.h"
#include "QBtColorDemo.h"
#include "QBtPicture.h"
#include "QBtShared.h"
#include "QBtEventsController.h"
#include "Parsery.h"
#include <QIcon>
#include <QLabel>
#include <QSlider>
#include <QGroupBox>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QSpinBox>
#include <QPushButton>
#include <QColorDialog>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigRotate::Caption       = QT_TR_NOOP( "Rotate" );
const char* const QBtConfigRotate::Suffix        = QT_TR_NOOP( " degree" );
const char* const QBtConfigRotate::RotationAngle = QT_TR_NOOP( "Angle of rotation:" );
const char* const QBtConfigRotate::Background    = QT_TR_NOOP( "Background color:" );
//const char* const QBtConfigRotate::Transparency  = QT_TR_NOOP( "transparency:" );
//const char* const QBtConfigRotate::Color         = QT_TR_NOOP( "color:" );


//*******************************************************************
// QBtConfigRotate                                       CONSTRUCTOR
//*******************************************************************
QBtConfigRotate::QBtConfigRotate( QWidget* const in_parent, const QString& in_fpath )
: QBtConfigBaseDialog ( in_parent, in_fpath )
, image_          ()
, angle_slider_   ( new QSlider( Qt::Horizontal ) )
, angle_spin_box_ ( new QSpinBox )
, color_btn_      ( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
, color_demo_     ( new QBtColorDemo )
//, trans_spin_box_ ( new QSpinBox )
{
   setWindowTitle( tr( Caption ) );
   setWindowIcon( QIcon( ":/img/rotate.png" ) );

   QLabel* const rotation_label   = new QLabel( tr( RotationAngle ) );
   QLabel* const background_label = new QLabel( tr( Background ) );
   //QLabel* const color_label      = new QLabel( tr( Color ) );
   //QLabel* const trans_label      = new QLabel( tr( Transparency ) );
   
   angle_slider_->setRange( 0, 360 );
   angle_slider_->setSingleStep( 10 );
   angle_slider_->setPageStep( 10 );
   angle_slider_->setTickPosition( QSlider::TicksBelow );
   angle_slider_->setValue( 0 );
   angle_spin_box_->setRange( 0, 360 );
   angle_spin_box_->setSingleStep( 1 );
   angle_spin_box_->setValue( 0 );
   angle_spin_box_->setSuffix( tr( Suffix ) );
   //trans_spin_box_->setRange( 0, 100 );
   //trans_spin_box_->setValue( 0 );
   //trans_spin_box_->setSuffix( "%" );

   QGridLayout* const grid = new QGridLayout;
   grid->addWidget( rotation_label  , 0, 0, 1, 3 );
   grid->addWidget( background_label, 0, 4, 1, 3 );
   grid->addWidget( angle_spin_box_ , 1, 1 );
   grid->addWidget( angle_slider_   , 2, 1 );
   //grid->addWidget( color_label     , 1, 4, 1, 1, Qt::AlignRight );
   grid->addWidget( color_demo_     , 1, 5, 1, 1, Qt::AlignLeft );
   grid->addWidget( color_btn_      , 1, 6, 1, 1, Qt::AlignLeft );
   //grid->addWidget( trans_label     , 2, 4, Qt::AlignRight );
   //grid->addWidget( trans_spin_box_ , 2, 5 );
   grid->setColumnMinimumWidth( 3, 10 );
   
   user_layout_->addLayout( grid );
   user_layout_->addStretch( 100 );

   connect( try_           , SIGNAL( clicked()         ), this, SLOT( try_clicked() ) );
   connect( accept_        , SIGNAL( clicked()         ), this, SLOT( accept_clicked() ) );
   connect( angle_slider_  , SIGNAL( valueChanged(int) ), this, SLOT( slider_changed(int) ) );
   connect( angle_spin_box_, SIGNAL( valueChanged(int) ), this, SLOT( angle_changed (int) ) );
   connect( color_btn_     , SIGNAL( clicked()         ), this, SLOT( change_color() ) );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::ParamsAnswer );
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigRotate

//*******************************************************************
// ~QBtConfigRotate                                       DESTRUCTOR
//*******************************************************************
QBtConfigRotate::~QBtConfigRotate()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigRotate

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigRotate::showEvent( QShowEvent* const e )
{
   QBtConfigBaseDialog::showEvent( e );
   
   const int dx = angle_spin_box_->width();
   angle_slider_->setMaximumWidth( dx );
   color_demo_->setMaximumWidth( dx );
   //trans_spin_box_->setMaximumWidth( dx );
   
   QBtEventsController::instance()->send_event( QBtEvent::ParamsRequest, "rotate" );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigRotate::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::ParamsAnswer:
         update_values( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "rotate", get_command() );
   }
}
// end of customEvent

//*******************************************************************
// update_values                                             PRIVATE
//*******************************************************************
void QBtConfigRotate::update_values( const QString& params )
{
   if( params.isEmpty() ) return;
   
   CBtRotateParser parser( params );
   angle_spin_box_->setValue( parser.angle() );
   color_demo_->background( parser.color() );
   //trans_spin_box_->setValue( parser.transparency() );
   update();
}
// end of update_values

//*******************************************************************
// get_command                                     PRIVATE inherited
//*******************************************************************
QString QBtConfigRotate::get_command() const
{
   CBtRotateParser parser;
   parser.angle( angle_spin_box_->value() );
   parser.color( color_demo_->background() );
   parser.transparency( 0 );
   return parser.command();
}
// end of get_command

//*******************************************************************
// slider_changed                                       PRIVATE slot
//*******************************************************************
void QBtConfigRotate::slider_changed( const int in_value )
{
   angle_spin_box_->setValue( in_value );
}
// end of slider_changed

//*******************************************************************
// angle_changed                                        PRIVATE slot
//*******************************************************************
void QBtConfigRotate::angle_changed( const int in_value )
{
   angle_slider_->setValue( in_value );
}
// end of angle_changed

//*******************************************************************
// change_color                                         PRIVATE slot
//*******************************************************************
void QBtConfigRotate::change_color()
{
   const QColor c = QColorDialog::getColor( color_demo_->background(), this );
   if( c.isValid() ) {
      color_demo_->background( c );
   }
}
// end of change_color

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigRotate::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.rotate( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();
}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigRotate::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "rotate", get_command() );
   accept(); 
}
// end of accept_clicked
