/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigWatermark.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 28.03.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include "Parsery.h"
#include "QBtConfigWatermark.h"
#include "QBtEventsController.h"
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QCheckBox>
#include <QSpinBox>
#include <QIcon>
#include <QGridLayout>
#include <QFileDialog>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigWatermark::Caption     = QT_TR_NOOP( "Watermark" );
const char* const QBtConfigWatermark::PathStr     = QT_TR_NOOP( "&Picture path:" );
const char* const QBtConfigWatermark::MarginStr   = QT_TR_NOOP( "&Margin:" );
const char* const QBtConfigWatermark::ScaleStr    = QT_TR_NOOP( "&Scale:" );
const char* const QBtConfigWatermark::XWidthStr   = QT_TR_NOOP( "&x-width" );
const char* const QBtConfigWatermark::YWidthStr   = QT_TR_NOOP( "&y-width" );
const char* const QBtConfigWatermark::GravityStr  = QT_TR_NOOP( "Gravity:" );
const char* const QBtConfigWatermark::Transparent = QT_TR_NOOP( "Transparency:" );
const char* const QBtConfigWatermark::NorthWest   = QT_TR_NOOP( "North-West" );
const char* const QBtConfigWatermark::North       = QT_TR_NOOP( "North" );
const char* const QBtConfigWatermark::NorthEast   = QT_TR_NOOP( "Nort-East" );
const char* const QBtConfigWatermark::West        = QT_TR_NOOP( "West" );
const char* const QBtConfigWatermark::Center      = QT_TR_NOOP( "Center" );
const char* const QBtConfigWatermark::East        = QT_TR_NOOP( "East" );
const char* const QBtConfigWatermark::SouthWest   = QT_TR_NOOP( "South-West" );
const char* const QBtConfigWatermark::South       = QT_TR_NOOP( "South" );
const char* const QBtConfigWatermark::SouthEast   = QT_TR_NOOP( "South-East" );


//*******************************************************************
// QBtConfigWatermark                                    CONSTRUCTOR
//*******************************************************************
QBtConfigWatermark::QBtConfigWatermark( QWidget* const parent, const QString& path )
: QBtConfigBaseDialog  ( parent, path )
, picture_path_        ( new QLineEdit )
, picture_path_button_ ( new QPushButton( QIcon( ":/img/folder.png" ), QString() ) )
, gravity_label_       ( new QLabel( tr( GravityStr ) ) )
, trans_label_         ( new QLabel( tr( Transparent ) ) )
, trans_spin_box_      ( new QSpinBox )
, nwg_rb_              ( new QRadioButton )
, ng_rb_               ( new QRadioButton )
, neg_rb_              ( new QRadioButton )
, wg_rb_               ( new QRadioButton )
, cg_rb_               ( new QRadioButton )
, eg_rb_               ( new QRadioButton )
, swg_rb_              ( new QRadioButton )
, sg_rb_               ( new QRadioButton )
, seg_rb_              ( new QRadioButton )
, margin_check_box_    ( new QCheckBox( tr( MarginStr ) ) )
, x_margin_label_      ( new QLabel( tr( XWidthStr ) ) )
, y_margin_label_      ( new QLabel( tr( YWidthStr ) ) )
, x_margin_spin_box_   ( new QSpinBox )
, y_margin_spin_box_   ( new QSpinBox )
, scale_check_box_     ( new QCheckBox( tr( ScaleStr ) ) )
, scale_spin_box_      ( new QSpinBox )
{
   setWindowTitle( tr( Caption ) );
   
   trans_spin_box_->setRange( 1, 100 );
   trans_spin_box_->setValue( 30 );
   trans_spin_box_->setSuffix( "%" );

   margin_check_box_->setCheckState( Qt::Unchecked );
   x_margin_spin_box_->setRange( 0, 10000 );
   x_margin_spin_box_->setValue( 4 );
   x_margin_spin_box_->setSuffix( "px" );
   y_margin_spin_box_->setRange( 0, 10000 );
   y_margin_spin_box_->setValue( 4 );
   y_margin_spin_box_->setSuffix( "px" );
   
   scale_spin_box_->setRange( 1, 100 );
   scale_spin_box_->setValue( 25 );
   scale_spin_box_->setSuffix( "%" );

   nwg_rb_->setToolTip( tr( NorthWest ) );
   ng_rb_ ->setToolTip( tr( North ) );
   neg_rb_->setToolTip( tr( NorthEast ) );
   wg_rb_ ->setToolTip( tr( West ) );
   cg_rb_ ->setToolTip( tr( Center ) );
   eg_rb_ ->setToolTip( tr( East ) );
   swg_rb_->setToolTip( tr( SouthWest ) );
   sg_rb_ ->setToolTip( tr( South ) );
   seg_rb_->setToolTip( tr( SouthEast ) );
   cg_rb_ ->setChecked( true );
   
   QLabel* const pic_path = new QLabel( tr( PathStr    ) );
   
   pic_path->setBuddy( picture_path_ );
   x_margin_label_->setBuddy( x_margin_spin_box_ );
   y_margin_label_->setBuddy( y_margin_spin_box_ );

   //------- PICTURE PATH ---------------------------------
   QHBoxLayout* const path_layout = new QHBoxLayout;
   path_layout->addWidget( pic_path );
   path_layout->addWidget( picture_path_);
   path_layout->addWidget( picture_path_button_ );

   //------- GRAVITY --------------------------------------
   QGridLayout* const grav_grid = new QGridLayout;
   grav_grid->addWidget( nwg_rb_, 0, 0 );
   grav_grid->addWidget( ng_rb_ , 0, 1 );
   grav_grid->addWidget( neg_rb_, 0, 2 );
   grav_grid->addWidget( wg_rb_ , 1, 0 );
   grav_grid->addWidget( cg_rb_ , 1, 1 );
   grav_grid->addWidget( eg_rb_ , 1, 2 );
   grav_grid->addWidget( swg_rb_, 2, 0 );
   grav_grid->addWidget( sg_rb_ , 2, 1 );
   grav_grid->addWidget( seg_rb_, 2, 2 );

   //------- TRANSPARENCY ---------------------------------
   QGridLayout* const trans_grid = new QGridLayout;
   trans_grid->addWidget( trans_spin_box_, 0, 1, 1, 1, Qt::AlignTop );
   trans_grid->setColumnMinimumWidth( 0, 20 );
   
   //------- X-MARGIN -------------------------------------
   QGridLayout* const x_grid = new QGridLayout;
   x_grid->addWidget( x_margin_label_   , 0, 0 );
   x_grid->addWidget( x_margin_spin_box_, 0, 1 );
   //------- Y-MARGIN -------------------------------------
   QGridLayout* const y_grid = new QGridLayout;
   y_grid->addWidget( y_margin_label_   , 0, 0 );
   y_grid->addWidget( y_margin_spin_box_, 0, 1 );

   QVBoxLayout* const margin_group = new QVBoxLayout;
   margin_group->addLayout( x_grid );
   margin_group->addLayout( y_grid );

   //------- SCALE ----------------------------------------
   QGridLayout* const scale_grid = new QGridLayout;
   scale_grid->addWidget( scale_spin_box_, 0, 1, 1, 1, Qt::AlignTop );
   scale_grid->setColumnMinimumWidth( 0, 20 );

   //------- TOOLS ----------------------------------------
   QGridLayout* const grid = new QGridLayout;
   grid->addLayout( path_layout      , 0,  0, 1, 11 );
   grid->addWidget( gravity_label_   , 1,  0, 1,  2 );
   grid->addWidget( trans_label_     , 1,  3, 1,  2 );
   grid->addWidget( margin_check_box_, 1,  6, 1,  2 );
   grid->addWidget( scale_check_box_ , 1,  9, 1,  2 );
   grid->addLayout( grav_grid        , 2,  1 );
   grid->addWidget( trans_spin_box_  , 2,  4, 1, 1, Qt::AlignTop );
   grid->addLayout( margin_group     , 2,  7 );
   grid->addWidget( scale_spin_box_  , 2, 10, 1, 1, Qt::AlignTop );
     
   grid->setColumnMinimumWidth( 0, 10 );
   grid->setColumnMinimumWidth( 3, 10 );
   grid->setColumnMinimumWidth( 6, 10 );
   grid->setColumnMinimumWidth( 9, 10 );

   //------- USER LAYOUT ----------------------------------
   user_layout_->addLayout( grid );
   user_layout_->addStretch();


   set_gravity_enabled( false );
      
   //------- SIGNAL/SLOT ----------------------------------
   connect( try_, SIGNAL( clicked()), this, SLOT( try_clicked()));
   connect( accept_, SIGNAL( clicked()), this, SLOT( accept_clicked()));
   connect( margin_check_box_, SIGNAL( stateChanged(int)), this, SLOT( margin_check_box_changed(int)));
   connect( scale_check_box_, SIGNAL( stateChanged(int)), this, SLOT( scale_check_box_changed(int)));
   connect( picture_path_, SIGNAL( textChanged(const QString&)), this, SLOT( picture_path_changed(const QString&)));
   connect( picture_path_button_, SIGNAL( clicked()), this, SLOT( picture_path_button_clicked()));

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::ParamsAnswer );
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigWatermark

//*******************************************************************
// ~QBtConfigWatermark                                         DESTRUCTOR
//*******************************************************************
QBtConfigWatermark::~QBtConfigWatermark()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigWatermark

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigWatermark::showEvent( QShowEvent* const e )
{
   QBtConfigBaseDialog::showEvent( e );
   QBtEventsController::instance()->send_event( QBtEvent::ParamsRequest, "watermark" );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigWatermark::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::ParamsAnswer:
         update_values( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "watermark", get_command() );
         break;
   }
}
// end of customEvent

//*******************************************************************
// update_values                                             PRIVATE
//*******************************************************************
void QBtConfigWatermark::update_values( const QString& params )
{
   CBtWatermarkParser parser( params );
   if( !parser.valid() ) return;

   picture_path_->setText( parser.path() );
   switch( parser.gravity() ) {
      case Magick::NorthWestGravity:
         nwg_rb_->setChecked( true );
         break;
      case Magick::NorthGravity:
         ng_rb_->setChecked( true );
         break;
      case Magick::NorthEastGravity:
         neg_rb_->setChecked( true );
         break;
      case Magick::WestGravity:
         wg_rb_->setChecked( true );
         break;
      case Magick::CenterGravity:
         cg_rb_->setChecked( true );
         break;
      case Magick::EastGravity:
         eg_rb_->setChecked( true );
         break;
      case Magick::SouthWestGravity:
          swg_rb_->setChecked( true );
         break;
      case Magick::SouthGravity:
         sg_rb_->setChecked( true );
         break;
      case Magick::SouthEastGravity:
         seg_rb_->setChecked( true );
         break;
      default:
         break;
   }
   trans_spin_box_->setValue( parser.transparency() );
   
   if(( parser.x_margin() != -1 ) && ( parser.y_margin() != -1 )) {
      margin_check_box_->setCheckState( Qt::Checked );
      x_margin_spin_box_->setValue( parser.x_margin() );
      y_margin_spin_box_->setValue( parser.y_margin() );
   }
   if( parser.scale() != -1 ) {
      scale_check_box_->setCheckState( Qt::Checked );
      scale_spin_box_->setValue( parser.scale() );
   }
}
// end of update_values

//*******************************************************************
// get_command                                               PRIVATE
//*******************************************************************
QString QBtConfigWatermark::get_command() const
{
   if( picture_path_->text().isEmpty()) return QString();
   
   CBtWatermarkParser parser;
   // sciezka do pliku
   parser.path( picture_path_->text() );
   // polozenie
   Magick::GravityType gravity = Magick::ForgetGravity;
        if( nwg_rb_->isChecked()) gravity = Magick::NorthWestGravity;
   else if( ng_rb_ ->isChecked()) gravity = Magick::NorthGravity;
   else if( neg_rb_->isChecked()) gravity = Magick::NorthEastGravity;
   else if( wg_rb_ ->isChecked()) gravity = Magick::WestGravity;
   else if( cg_rb_ ->isChecked()) gravity = Magick::CenterGravity;
   else if( eg_rb_ ->isChecked()) gravity = Magick::EastGravity;
   else if( swg_rb_->isChecked()) gravity = Magick::SouthWestGravity;
   else if( sg_rb_ ->isChecked()) gravity = Magick::SouthGravity;
   else if( seg_rb_->isChecked()) gravity = Magick::SouthEastGravity;
   parser.gravity( gravity );
   // przezroczystosc
   parser.transparency( trans_spin_box_->value() );
   // marginesy
   if( Qt::Checked == margin_check_box_->checkState() ) {
      parser.x_margin( x_margin_spin_box_->value() );
      parser.y_margin( y_margin_spin_box_->value() );
   }
   // transform czy scale
   if( Qt::Checked == scale_check_box_->checkState() ) {
      parser.scale( scale_spin_box_->value() );
   }
   return parser.command();
}
// end of get_command

//*******************************************************************
// set_margin_enabled                                        PRIVATE
//*******************************************************************
void QBtConfigWatermark::set_margin_enabled( const bool state )
{
   x_margin_label_->setEnabled( state );
   y_margin_label_->setEnabled( state );
   x_margin_spin_box_->setEnabled( state );
   y_margin_spin_box_->setEnabled( state );
}
// end of set_margin_enabled

//*******************************************************************
// set_scale_enabled                                         PRIVATE
//*******************************************************************
void QBtConfigWatermark::set_scale_enabled( const bool state )
{
   scale_spin_box_->setEnabled( state );
}
// end of set_scale_enabled

//*******************************************************************
// set_gravity_enabled                                       PRIVATE
//*******************************************************************
void QBtConfigWatermark::set_gravity_enabled( const bool state )
{
   gravity_label_->setEnabled( state );
   nwg_rb_->setEnabled( state );
   ng_rb_ ->setEnabled( state );
   neg_rb_->setEnabled( state );
   wg_rb_ ->setEnabled( state );
   cg_rb_ ->setEnabled( state );
   eg_rb_ ->setEnabled( state );
   swg_rb_->setEnabled( state );
   sg_rb_ ->setEnabled( state );
   seg_rb_->setEnabled( state );

   trans_label_->setEnabled( state );
   trans_spin_box_->setEnabled( state );

   if( state ) {
      set_margin_enabled( Qt::Checked == margin_check_box_->checkState() );
      set_scale_enabled( Qt::Checked == scale_check_box_->checkState() );
   }
   else {
      set_margin_enabled( false );
      set_scale_enabled( false );
   }
   margin_check_box_->setEnabled( state );
   scale_check_box_->setEnabled( state );
}
// end of set_gravity_enabled

//*******************************************************************
// margin_check_box_changed                             PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::margin_check_box_changed( int state )
{
   switch( state ) {
      case Qt::Checked:
         margin_check_box_->setCheckState( Qt::Checked );
         set_margin_enabled( true );
         break;
      case Qt::Unchecked:
         margin_check_box_->setCheckState( Qt::Unchecked );
         set_margin_enabled( false );
         break;
   }
}
// end of margin_check_box_changed

//*******************************************************************
// scale_check_box_changed                              PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::scale_check_box_changed( int state )
{
   switch( state ) {
      case Qt::Checked:
         scale_check_box_->setCheckState( Qt::Checked );
         set_scale_enabled( true );
         break;
      case Qt::Unchecked:
         scale_check_box_->setCheckState( Qt::Unchecked );
         set_scale_enabled( false );
         break;
   }
}
// end of scale_check_box_changed

//*******************************************************************
// picture_path_changed                                 PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::picture_path_changed( const QString& path )
{
   set_gravity_enabled( !path.isEmpty() );
}
// end of picture_path_changed

//*******************************************************************
// picture_path_button_clicked                          PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::picture_path_button_clicked()
{
   const QString path = QFileDialog::getOpenFileName( this );
   if( !path.isEmpty() ) {
      picture_path_->setText( path );
   }
}
// end of picture_path_button_cliked

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.watermark( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();
}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigWatermark::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "watermark",get_command() );
   accept(); 
}
// end of accept_clicked
