/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtEventsController.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 28.01.2009
 *******************************************************************/
 
/*------- include files:
-------------------------------------------------------------------*/
#include "QBtEventsController.h"
#include <QApplication>
#ifdef WITH_MUTEX
#include <QMutexLocker>
#endif
#include <QtDebug>
using namespace std;

//*******************************************************************
// QBtEventsController                           CONSTRUCTOR private
//*******************************************************************
QBtEventsController::QBtEventsController() : QObject()
{
   setParent( qApp );
}
// end of EventsControler

//*******************************************************************
// ~EventsContoller                               DESTRUCTOR private
//*******************************************************************
QBtEventsController::~QBtEventsController()
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif
   evreg_.clear();
}
// end of ~QBtEventsController

//*******************************************************************
// instance                                                   PUBLIC
//*******************************************************************
QBtEventsController* QBtEventsController::instance()
{
   static QBtEventsController* instance = 0;
   if( !instance ) instance = new QBtEventsController;
   return instance;
}
// end of instance

//*******************************************************************
// append                                                     PUBLIC
//-------------------------------------------------------------------
// Zarejestrowanie chetnego do odbierania wskazanego sygnalu.
//*******************************************************************
bool QBtEventsController::append( QObject* const in_receiver, const int in_event_id )
{
#ifdef WITH_MUTEC
   QMutexLocker locker( &mutex_ );
#endif
   bool retval = true;

   // Jesli taka pozycja juz istnieje to nie zapamietujemy jej ponownie.
   if( exists( in_receiver, in_event_id ) ) {
      retval = false;
   }
   // Jesli nie ma takiej pozycji to ja zapamietujemy w rejestrze.
   else {
      EventsRegister::iterator it = evreg_.find( in_event_id );
      if( it != evreg_.end() ) {
         retval = it->second.insert( in_receiver ).second;
      }
   }
   return retval;
}
// end of append

//*******************************************************************
// remove                                                     PUBLIC
//*******************************************************************
void QBtEventsController::remove( QObject* const in_receiver )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif

   if( !evreg_.empty() ) {
      EventsRegister::iterator it = evreg_.begin();
      const EventsRegister::iterator end = evreg_.end();
      while( it != end ) {
         it->second.erase( in_receiver );
         ++it;
      }
   }
}
// end of remove

//*******************************************************************
// exists                                                    PRIVATE
//*******************************************************************
bool QBtEventsController::exists( QObject* const in_receiver, const int in_event_id )
{
   bool retval = false;

   const EventsRegister::const_iterator it = evreg_.find( in_event_id );
   if( it != evreg_.end() ) {
      if( it->second.find( in_receiver ) != it->second.end() ) {
         retval = true;
      }
   }
   else {
      // Dopisuje do rejestru TYLKO identyfikator sygnalu (nie obiekt).
      evreg_.insert( EventsRegister::value_type( in_event_id, Objects() ) );
   }
   return retval;
}
// end of exists

//*******************************************************************
// send_event                                                 PUBLIC
//*******************************************************************

// WERSJA BEZ DANYCH
void QBtEventsController::send_event( const int in_event_id )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif
   const EventsRegister::const_iterator rit = evreg_.find( in_event_id );
   if( rit != evreg_.end() ) {
      Objects::const_iterator it = rit->second.begin();
      const Objects::const_iterator end = rit->second.end();
      while( it != end ) {
         QApplication::postEvent( *it, new QBtEvent( in_event_id ) );
         ++it;
      }
   }
}
// WERSJA Z 1 DANA
void QBtEventsController::send_event(  const int in_event_id,
                                       const QVariant in_val )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif

   const EventsRegister::const_iterator rit = evreg_.find( in_event_id );
   if( rit != evreg_.end() ) {
      Objects::const_iterator it = rit->second.begin();
      const Objects::const_iterator end = rit->second.end();
      while( it != end ) {
         QApplication::postEvent( *it, new QBtEvent( in_event_id, in_val ) );
         ++it;
      }
   }
}
// WERSJA Z 2 DANYMI
void QBtEventsController::send_event(  const int in_event_id,
                                       const QVariant in_val1,
                                       const QVariant in_val2 )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif

   const EventsRegister::const_iterator rit = evreg_.find( in_event_id );
   if( rit != evreg_.end() ) {
      Objects::const_iterator it = rit->second.begin();
      const Objects::const_iterator end = rit->second.end();
      while( it != end ) {
         QApplication::postEvent( *it, new QBtEvent( in_event_id, in_val1, in_val2 ) );
         ++it;
      }
   }
}
// WERSJA Z 3 DANYMI
void QBtEventsController::send_event(  const int in_event_id,
                                       const QVariant in_val1,
                                       const QVariant in_val2,
                                       const QVariant in_val3 )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif

   const EventsRegister::const_iterator rit = evreg_.find( in_event_id );
   if( rit != evreg_.end() ) {
      Objects::const_iterator it = rit->second.begin();
      const Objects::const_iterator end = rit->second.end();
      while( it != end ) {
         QApplication::postEvent( *it, new QBtEvent( in_event_id, in_val1, in_val2, in_val3 ) );
         ++it;
      }
   }
}
// WERSJA Z 4 DANYMI
void QBtEventsController::send_event(  const int in_event_id,
                                       const QVariant in_val1,
                                       const QVariant in_val2,
                                       const QVariant in_val3,
                                       const QVariant in_val4 )
{
#ifdef WITH_MUTEX
   QMutexLocker locker( &mutex_ );
#endif

   const EventsRegister::const_iterator rit = evreg_.find( in_event_id );
   if( rit != evreg_.end() ) {
      Objects::const_iterator it = rit->second.begin();
      const Objects::const_iterator end = rit->second.end();
      while( it != end ) {
         QApplication::postEvent( *it, new QBtEvent( in_event_id, in_val1, in_val2, in_val3, in_val4 ) );
         ++it;
      }
   }
}
// end of send_event
