/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtMagic.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 10.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include "QBtEventsController.h"
#include "QBtMagic.h"
#include "CBtString.h"
#include "Parsery.h"
#include <iostream>
#include <QFileInfo>
#include <QMessageBox>
#include <QColor>
#include <QtDebug>
using namespace std;

//*******************************************************************
// QBtMagic                                              CONSTRUCTOR
//*******************************************************************
QBtMagic::QBtMagic( QWidget* const in_parent ) : QObject()
, parent_ ( in_parent )
, image_  ()
, fdir_   ( QString() )
, fname_  ( QString() )
, fext_   ( QString() )
, ok_     ( false    )
, qimage_ ()
, visual_ ( false )
{}
QBtMagic::QBtMagic( const QBtMagic& rhs )
: QObject()
, parent_( rhs.parent_ )
, image_ ( rhs.image_  )
, fdir_  ( rhs.fdir_   )
, fname_ ( rhs.fname_  )
, fext_  ( rhs.fext_   )
, ok_    ( rhs.ok_     )
, visual_( rhs.visual_ )
{}
// end of QBtMagic

//*******************************************************************
// ~QBtMagic                                              DESTRUCTOR
//*******************************************************************
QBtMagic::~QBtMagic()
{}
// end of ~QBtMagic

//*******************************************************************
// operator=                                                  PUBLIC
//*******************************************************************
QBtMagic& QBtMagic::operator=( const QBtMagic& rhs )
{
   if( this != &rhs ) {
      parent_ = rhs.parent_;
      image_  = rhs.image_;
      fdir_   = rhs.fdir_;
      fname_  = rhs.fname_;
      fext_   = rhs.fext_;
      ok_     = rhs.ok_;
      visual_ = rhs.visual_;
   }
   return *this;
}
// end of operator=

//*******************************************************************
// read                                                       PUBLIC
//*******************************************************************
bool QBtMagic::read( const QString& fpath )
{
   try {
      image_.read( fpath.toStdString() );
      remember_fpath( fpath );
      image_.type( Magick::TrueColorType );
      return ( ok_ = true );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage,
                                                   QString( e.what() ) );
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage,
                                                   QString( e.what() ) );
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage,
                                                   QString( "???" ) );
   }
   return ( ok_ = false );
}
// end of read

//*******************************************************************
// write                                                      PUBLIC
//*******************************************************************
bool QBtMagic::write()
{
   return write2dir( fdir_ );
}
//*******************************************************************
// write2dir                                                  PUBLIC
//*******************************************************************
bool QBtMagic::write2dir( const QString& dname )
{
   if( !ok_ ) return false;
   
   try {
      QString fpath = dname;
      fpath += '/';
      fpath += fname_;
      fpath += '.';
      fpath += fext_;
      image_.write( fpath.toStdString() );
      return ( ok_ = true );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
   }
   return ( ok_ = false );
}
// end of write2dir

//*******************************************************************
// write2file                                                 PUBLIC
//*******************************************************************
bool QBtMagic::write2file( const QString& fname )
{
   if( !ok_ ) return false;
   
   try {
      image_.write( fname.toStdString() );
      return ( ok_ = true );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
   }
   
   return ( ok_ = false );
}
// end of write2file

//*******************************************************************
// remember_fpath                                            PRIVATE
//*******************************************************************
void QBtMagic::remember_fpath( const QString& fpath )
{
   fdir_ = fname_ = fext_ = QString();

   const QFileInfo info( fpath );
   fdir_  = info.absolutePath();
   fname_ = info.completeBaseName();
   fext_  = info.suffix();
}
// end of remember_fpath

//*******************************************************************
// qimage                                                     PUBLIC
//*******************************************************************
QImage& QBtMagic::qimage()
{
   if( !ok_ ) return ( qimage_ = QImage() ); 

   qimage_ = QImage( image_.columns(), image_.rows(), QImage::Format_RGB32);
   Magick::ColorRGB rgb;

   const int qheight = qimage_.height();
   const int qwidth  = qimage_.width();
   
   for( int row = 0; row < qheight; ++row ) {
      const Magick::PixelPacket* const  pixels = image_.getConstPixels( 0, row, qwidth, 1 );
      for( int col = 0; col < qwidth; ++col ) {
         rgb = *( pixels + col );
         const QRgb qrgb = qRgb( static_cast<int>( 255 * rgb.red  () ),
                                 static_cast<int>( 255 * rgb.green() ),
                                 static_cast<int>( 255 * rgb.blue () ) );
         qimage_.setPixel( col, row, qrgb );
      }
   }
  
   return qimage_;
}
// end of qimage

//*******************************************************************
// process                                                    PUBLIC
//-------------------------------------------------------------------
// Funkcja analizujaca przyslane polecenie i wywolujaca funkcje
// stosowne do wykonania zadania okreslonego przez polecenie.
//*******************************************************************
bool QBtMagic::process( const QString& cmd )
{
   const int idx = cmd.indexOf( '#' );
   if( -1 == idx ) return false;
   
   const QString command = cmd.left( idx );
   const QString params  = cmd.mid( idx + 1 );
   return process( command, params );
}
bool QBtMagic::process( const QString& command, const QString& params )
{
  try {
           if( "format"    == command ) set_format( params );
      else if( "watermark" == command ) watermark ( params );
      else if( "resize"    == command ) resize    ( params );
      else if( "annotate"  == command ) annotate  ( params );
      else if( "grayscale" == command ) grayscale ( params );
      else if( "negate"    == command ) negate    ( params );
      else if( "border"    == command ) border    ( params );
      else if( "rotate"    == command ) rotate    ( params );
      else if( "normalize" == command ) normalize ( params );
      else if( "enhance"   == command ) enhance   ( params );
      else if( "equalize"  == command ) equalize  ( params );
      else return false;
      return true;
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
   }

   return false;
}
// end of process

//*******************************************************************
// clear                                                      PUBLIC
//*******************************************************************
void QBtMagic::clear()
{
   image_ = Magick::Image();
   ok_ = false;
}
// end of clear

//*******************************************************************
// set_format                                                 PUBLIC
//*******************************************************************
void QBtMagic::set_format( const QString& fext )
{
   fext_ = fext;
   image_.magick( fext_.toStdString() );
}
// end of set_format

//*******************************************************************
// get_gravity_type                                          PRIVATE
//-------------------------------------------------------------------
// Zamienia opis slowny na wartosc liczbowa.
//*******************************************************************
Magick::GravityType QBtMagic::get_gravity_type( const QString& text ) const
{
   if( text == "NorthWestGravity" ) return Magick::NorthWestGravity;
   if( text == "NorthGravity"     ) return Magick::NorthGravity;
   if( text == "NorthEastGravity" ) return Magick::NorthEastGravity;
   if( text == "WestGravity"      ) return Magick::WestGravity;
   if( text == "CenterGravity"    ) return Magick::CenterGravity;
   if( text == "EastGravity"      ) return Magick::EastGravity;
   if( text == "SouthWestGravity" ) return Magick::SouthWestGravity;
   if( text == "SouthGravity"     ) return Magick::SouthGravity;
   if( text == "SouthEastGravity" ) return Magick::SouthEastGravity;

   return Magick::ForgetGravity;
}
// end of get_gravity_type

//*******************************************************************
// ustaw_przezroczystosc                                     PRIVATE
//*******************************************************************
void QBtMagic::ustaw_przezroczystosc( Magick::Image& image, const double value ) const
{
   image.modifyImage();
   Magick::Pixels view( image );
   const unsigned int n  = image.columns();
   const unsigned int m = image.rows();
   
   Magick::PixelPacket* pixels = view.get( 0, 0, n, m );
   Magick::Color color;
   
   for( unsigned int i = 0; i < n; ++i ) {
      for( unsigned int j = 0; j < m; ++j ) {
         color = *pixels;
         if( color.alphaQuantum() < value ) {
            color.alphaQuantum( value );
            *pixels = color;
         }
         ++pixels;
      }
   }
   view.sync();
}
// end of ustaw_przezroczystosc

//*******************************************************************
// grayscale                                                  PUBLIC
//*******************************************************************
void QBtMagic::grayscale( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   if( command == "yes" ) {
      image_.type( Magick::GrayscaleType );
   }
}
// end of grayscale

//*******************************************************************
// negate                                                     PUBLIC
//*******************************************************************
void QBtMagic::negate( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   if( command == "yes" ) {
      image_.negate();
   }
}
// end of negate

//*******************************************************************
// normalize                                                  PUBLIC
//*******************************************************************
void QBtMagic::normalize( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   if( command == "yes" ) {
      image_.modifyImage();
      image_.normalize();
   }
}
// end of normalize

//*******************************************************************
// enhance                                                    PUBLIC
//*******************************************************************
void QBtMagic::enhance( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   if( command == "yes" ) {
      image_.enhance();
   }
}
// end of enhance

//*******************************************************************
// equalize                                                   PUBLIC
//*******************************************************************
void QBtMagic::equalize( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   
   if( command == "yes" ) {
      image_.equalize();
   }
}
// end of equalize

//*******************************************************************
// border                                                     PUBLIC
//*******************************************************************
void QBtMagic::border( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   
   if( command.isEmpty() ) return;

   CBtBorderParser parser( command );
   CArea top_area, bottom_area, left_area, right_area;
   
   vector<CArea> areas;
   areas.push_back( CArea() );
   areas.push_back( CArea() );
   areas.push_back( CArea() );
   areas.push_back( CArea() );

   if( parser.top().acceptable() ) {
      top_area.color() = parser.top().im_color();
      top_area.set_xlt( 0 );
      top_area.set_ylt( 0 );
      top_area.set_xrb( width() - 1 );
      top_area.set_yrb( parser.top().width() - 1 );
      areas[ parser.top().order() - 1 ] = top_area;
   }
   if( parser.bottom().acceptable() ) {
      bottom_area.color() = parser.bottom().im_color();
      bottom_area.set_xlt( 0 );
      bottom_area.set_ylt( height() - parser.bottom().width() - 1 );
      bottom_area.set_xrb( width() - 1 );
      bottom_area.set_yrb( height() - 1 );
      areas[ parser.bottom().order() - 1 ] = bottom_area;
   }
   if( parser.left().acceptable() ) {
      left_area.color() = parser.left().im_color();
      left_area.set_xlt( 0 );
      left_area.set_ylt( 0 );
      left_area.set_xrb( parser.left().width() - 1 );
      left_area.set_yrb( height() - 1 );
      areas[ parser.left().order() - 1 ] = left_area;
   }
   if( parser.right().acceptable() ) {
      right_area.color() = parser.right().im_color();
      right_area.set_xlt( width() - parser.right().width() - 1 );
      right_area.set_ylt( 0 );
      right_area.set_xrb( width() - 1 );
      right_area.set_yrb( height() - 1 );
      areas[ parser.right().order() - 1 ] = right_area;
   }

   for( int i = 0; i < 4; ++i ) {
      if( areas[i].valid() ) {
         list<Magick::Drawable> data = list<Magick::Drawable>();
         data.push_back( Magick::DrawableFillColor( areas[i].color() ) );
         data.push_back( Magick::DrawableStrokeColor( areas[i].color() ) );
         data.push_back( Magick::DrawableRectangle( areas[i].xlt(),
                                                    areas[i].ylt(),
                                                    areas[i].xrb(),
                                                    areas[i].yrb() ) );
         image_.draw( data );
      }
   }
}
// end of border

//*******************************************************************
// watermark                                                  PUBLIC
//*******************************************************************
void QBtMagic::watermark( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }

   CBtWatermarkParser parser( command );
   QString             fpath   = parser.path();
   int                 x       = parser.x_margin();
   int                 y       = parser.y_margin();
   int                 scale   = parser.scale();
   double              trans   = parser.transparency() * MaxRGB * 0.01;
   Magick::GravityType gravity = parser.gravity();
   
   //--- ODCZYT I PRZETWORZENIE OBRAZU ZNAKU WODNEGO ----------------
   Magick::Image water_image( fpath.toStdString() );
   ustaw_przezroczystosc( water_image, trans );

   //--- SKALOWANIE ---------------------------------------
   if( scale > 0 ) {
      const int scaled_width  = static_cast<int>( image_.columns() * scale * 0.01L );
      const int scaled_height = static_cast<int>( image_.rows()    * scale * 0.01L );

      Magick::Geometry scaled_geometry( scaled_width, scaled_height );
      scaled_geometry.greater( true );
      water_image.transform( scaled_geometry );
   }

   if(( x >= 0 ) && ( y >= 0 )) {
      //--- PRZEZROCZYSTY OBRAZ TLA --------------------------
      const int dst_width  = water_image.columns() + ( x << 1 );
      const int dst_height = water_image.rows()    + ( y << 1 );
      
      const Magick::Geometry dst_geometry( dst_width, dst_height );
      Magick::Image dst_image( dst_geometry, Magick::Color( 0, 0, 0, MaxRGB ));
      dst_image.composite( water_image, Magick::CenterGravity, Magick::OverCompositeOp );
   
      //--- OBRAZ WYNIKOWY -----------------------------------
      image_.composite( dst_image, gravity, Magick::OverCompositeOp );
   }
   else {
      //--- OBRAZ WYNIKOWY -----------------------------------
      image_.composite( water_image, gravity, Magick::OverCompositeOp );
   }
}
// end of watermark

//*******************************************************************
// rotate                                                     PUBLIC
//*******************************************************************
void QBtMagic::rotate( const int degree )
{
   image_.rotate( degree );
}
void QBtMagic::rotate( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   
   CBtRotateParser parser( command );
   image_.backgroundColor( parser.im_color() );
   image_.rotate( parser.angle() );  
}
// end of rotate

//*******************************************************************
// annotate                                                   PUBLIC
//-------------------------------------------------------------------
// Nalozenie na aktualny obraz napisu 'text'.
//*******************************************************************
void QBtMagic::annotate( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   
   CBtAnnotateParser parser( command );
   if( !parser.valid() ) return;

   bool    is_pic = false;
   QString data   = QString();

   if( parser.fpath().isEmpty() ) {
      data = parser.text();
   }
   else {
      is_pic = true;
      data = parser.fpath();
   }
   const Magick::Color color    = parser.im_color();
   const int           x_margin = parser.x_margin();
   const int           y_margin = parser.y_margin();

   const int pic_width  = image_.columns();
   const int pic_height = image_.rows();
   if( is_pic ) {
      const Magick::Image logo( data.toStdString() );
      const int logo_height = logo.rows();

      const int dst_width  = pic_width - ( 2 * x_margin );
      const int dst_height = logo_height;
      const Magick::Geometry dst_geometry( dst_width, dst_height );
      Magick::Image dst_image( dst_geometry, color );
      dst_image.composite( logo, Magick::CenterGravity, Magick::OverCompositeOp );

      const int x_offset = x_margin;
      const int y_offset = pic_height - dst_height - y_margin;
      image_.composite( dst_image, x_offset, y_offset, Magick::OverCompositeOp );
   }
   else {
      const int dst_width  = pic_width - ( 2 * x_margin );
      const int dst_height = 24;
      const Magick::Geometry dst_geometry( dst_width, dst_height );
      Magick::Image dst_image( dst_geometry, color );
      dst_image.strokeAntiAlias( true );
      dst_image.annotate( data.toStdString(), Magick::CenterGravity );

      const int x_offset = x_margin;
      const int y_offset = pic_height - dst_height - y_margin;
      image_.composite( dst_image, x_offset, y_offset, Magick::OverCompositeOp );
   }
}
// end of annotate

//*******************************************************************
// resize                                                     PUBLIC
//*******************************************************************
void QBtMagic::resize( const QString& command )
{
   if( visual_ ) {
      QBtEventsController::instance()->send_event( QBtEvent::Operation );
   }
   
   CBtResizeParser parser( command );
   if( !parser.valid() ) return;

   Magick::Geometry geometry( 0, 0 );
   geometry.greater( true );
   if( parser.percent() != -1 ) {
      geometry.width( parser.percent() );
      geometry.height( parser.percent() );
      geometry.percent( true );
      geometry.aspect( true );
   }
   else {
      geometry.width( parser.width() );
      geometry.height( parser.height() );
      geometry.percent( false );
      geometry.aspect( false );
   }

   if( parser.scale() ) image_.scale( geometry );
   else                 image_.transform( geometry );   
}
// end of resize
