/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBProcessDialog.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 18.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtProcessDialog.h"
#include "QBtMagic.h"
#include "QBtEventsController.h"
#include <QDir>
#include <QLabel>
#include <QRegExp>
#include <QSpinBox>
#include <QFileInfo>
#include <QCheckBox>
#include <QLineEdit>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QProgressBar>
#include <QApplication>
#include <QDesktopWidget>
#include <QMessageBox>
//#include <QtDebug>


/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtProcessDialog::Caption     = QT_TR_NOOP( "Image processing" );
const char* const QBtProcessDialog::Start       = QT_TR_NOOP( "&Start" );
const char* const QBtProcessDialog::Cancel      = QT_TR_NOOP( "&Cancel" );
const char* const QBtProcessDialog::Break       = QT_TR_NOOP( "&Break" );
const char* const QBtProcessDialog::Progress    = QT_TR_NOOP( "Progress:" );
const char* const QBtProcessDialog::Source      = QT_TR_NOOP( "Source:" );
const char* const QBtProcessDialog::Destination = QT_TR_NOOP( "Destination:" );
const char* const QBtProcessDialog::Rename      = QT_TR_NOOP( "&rename" );
const char* const QBtProcessDialog::StartNumber = QT_TR_NOOP( "Start number:" );
const char* const QBtProcessDialog::RenameInfo  =
 QT_TR_NOOP( "<qt><font color=red>Rename muster have invalid format!</font><br>"
             "Muster should contain valid file name characters and format tags:<br>"
             "%f - oryginal file name,<br>"
             "%d[d..d] - calculated number.<br>"
             "Muster <u>must</u> contain one or both format tags.</qt>" );

const QString QBtProcessDialog::FileTag   = "%f";
const QString QBtProcessDialog::NumberTag = "%d";


//*******************************************************************
// QBtProcessDialog                                      CONSTRUCTOR
//*******************************************************************
QBtProcessDialog::QBtProcessDialog( QWidget* const parent,
                                    const Strings& files,
                                    const QString& dst_dir,
                                    const QStringList& commands )
: QDialog          ( parent   )
, files_           ( files    )
, dst_dir_         ( dst_dir  )
, commands_        ( commands )
, start_           ( new QPushButton( tr( Start  ) ) )
, cancel_          ( new QPushButton( tr( Cancel ) ) )
, progress_        ( new QProgressBar )
, source_          ( new QLineEdit )
, destination_     ( new QLineEdit )
, rename_cbox_     ( new QCheckBox( tr( Rename ) ) )
, rename_muster_   ( new QLineEdit )
, number_label_    ( new QLabel( tr( StartNumber ) ) )
, number_spin_box_ ( new QSpinBox )
, continue_        ( true )
, n_               ( 0 )
{
   setWindowTitle( tr( Caption ) );

   progress_->setRange( 0, files_.size() * commands_.size() );
   progress_->setValue( n_ );
   progress_->setTextVisible( false );
   source_->setReadOnly( true );
   destination_->setReadOnly( true );
   rename_cbox_->setCheckState( Qt::Unchecked );
   rename_checked( Qt::Unchecked );
   number_spin_box_->setRange( 1, 10000 );
   number_spin_box_->setValue( 1 );

   QLabel* const progress_label    = new QLabel( tr( Progress ) );
   QLabel* const source_label      = new QLabel( tr( Source ) );
   QLabel* const destination_label = new QLabel( tr( Destination ) );
   //QLabel* const oper_label     = new QLabel( tr( Operation ) );
   
   QGridLayout* const grid = new QGridLayout;
   grid->addWidget( progress_label   , 0, 0 );
   grid->addWidget( progress_        , 0, 1 );
   grid->addWidget( source_label     , 1, 0 );
   grid->addWidget( source_          , 1, 1 );
   grid->addWidget( destination_label, 2, 0 );
   grid->addWidget( destination_     , 2, 1 );

   QHBoxLayout* const btn_layout = new QHBoxLayout;
   btn_layout->addWidget( rename_cbox_ );
   btn_layout->addWidget( rename_muster_ );
   btn_layout->addStretch();
   btn_layout->addWidget( number_label_ );
   btn_layout->addWidget( number_spin_box_ );
   btn_layout->addStretch();
   btn_layout->addWidget( start_ );
   btn_layout->addWidget( cancel_ );
   
   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addLayout( grid );
   main_layout->addLayout( btn_layout );
   setLayout( main_layout );

   connect( start_ , SIGNAL( clicked() ), this, SLOT( start_clicked() ) );
   connect( cancel_, SIGNAL( clicked() ), this, SLOT( cancel_clicked() ) );
   connect( rename_cbox_, SIGNAL( stateChanged(int) ), this, SLOT( rename_checked(int) ) );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::Operation );
}
// end of QBtProcessDialog

//*******************************************************************
// ~QBtProcessDialog                                      DESTRUCTOR
//*******************************************************************
QBtProcessDialog::~QBtProcessDialog()
{
   QBtEventsController* const evc = QBtEventsController::instance();
   evc->remove( this );
}
// end of ~QBtProcessDialog

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtProcessDialog::showEvent( QShowEvent* const e )
{
   QDialog::showEvent( e );

   const int scr_width  = qApp->desktop()->availableGeometry().width();
   const int width = scr_width * ( 40.0 / 100.0 );
   resize( width, size().height() );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtProcessDialog::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::Operation:
         update_operation_info();
         break;
   }
}
// end of customEvent

//*******************************************************************
// start_clicked                                        PRIVATE slot
//*******************************************************************
void QBtProcessDialog::start_clicked()
{
   if( rename_cbox_->checkState() == Qt::Checked ) {
      if( !rename_muster_valid() ) {
         QMessageBox::warning( this, tr( Caption ), tr( RenameInfo ) );
         rename_muster_->setFocus( Qt::OtherFocusReason );
         return;
      }
   }
   start_->setEnabled( false );
   cancel_->setText( tr( Break ) );
   rename_cbox_->setEnabled( false );
   rename_muster_->setEnabled( false );
   number_label_->setEnabled( false );
   number_spin_box_->setEnabled( false );
   run();
}
// end of start_clicked

//*******************************************************************
// cancel_clicked                                       PRIVATE slot
//*******************************************************************
void QBtProcessDialog::cancel_clicked()
{
   if( start_->isVisible() ) {
      accept();
   }
   else {
      continue_ = false;
      cancel_->setEnabled( false );
      QApplication::processEvents();
   }
}
// end of cancel_clicked

//*******************************************************************
// run                                                       PRIVATE
//*******************************************************************
void QBtProcessDialog::run()
{
   QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
   
   QBtMagic magic( this );
   magic.visual_on();

   bool ok = true;
   int counter = number_spin_box_->value();

   const QString muster = rename_muster_->text();
   const QStringList tagi = tags( muster ); 
   
   const int cmd_count = commands_.size();
   Strings::const_iterator it = files_.begin();
   const Strings::const_iterator end = files_.end();
   
   while( continue_ && ( it != end )) {
      QApplication::processEvents();
      
      const QString dst_file_path = dst_fpath( *it, dst_dir_, muster, tagi, counter );
      update_src_info( *it );
      update_dst_info( dst_file_path );
      
      if( !magic.read( *it ) ) {
         if( ok ) ok = false;
      }
      QApplication::processEvents();
      //...................................................
      for( int i = 0; continue_ && ( i < cmd_count ); ++i ) {
         if( !magic.process( commands_[i] ) ) {
            if( ok ) ok = false;
         }
         QApplication::processEvents();
      }
      //...................................................
      if( !magic.write2file( dst_file_path ) ) {
         if( ok ) ok = false;
      }
      QApplication::processEvents();
      ++it;
   }

   if( !continue_ ) {
      ok ? accept() : reject();
   }
   accept();
}
// end of run

//*******************************************************************
// update_src_info                                           PRIVATE
//*******************************************************************
void QBtProcessDialog::update_src_info( const QString& text )
{
   source_->setText( text );
   QApplication::processEvents();
}
// end of update_src_info

//*******************************************************************
// update_dst_info                                           PRIVATE
//*******************************************************************
void QBtProcessDialog::update_dst_info( const QString& text )
{
   destination_->setText( text );
   QApplication::processEvents();
}
// end of update_src_info

//*******************************************************************
// update_operation_info                                     PRIVATE
//*******************************************************************
void QBtProcessDialog::update_operation_info()
{
   progress_->setValue( ++n_ );
   progress_->update();
   QApplication::processEvents();   
}
// end of update_operation_info

//*******************************************************************
// rename_checked                                       PRIVATE slot
//*******************************************************************
void QBtProcessDialog::rename_checked( const int state )
{
   rename_muster_->setEnabled( state == Qt::Checked );
}
// end of rename_checked

//*******************************************************************
// dst_fpath
//*******************************************************************
QString QBtProcessDialog::dst_fpath( const QString&     src_fpath,
                                     const QString&     dst_dir,
                                     const QString&     muster,
                                     const QStringList& tags,
                                     int&               counter )
{
   const QFileInfo info( src_fpath );
   const QString fext  = info.suffix();
   QString fname = info.completeBaseName();
   
   if( !tags.empty() ) {
      QString tmp = muster;
      const int n = tags.size();
      for( int i = 0; i < n; ++i ) {
         if( tags[i].endsWith( "f" ) ) {
            tmp.replace( tags[i], fname );
         }
         else if( tags[i].endsWith( "d" ) ) {
            const int fld_size = tags[i].mid( 1, tags[i].size() - 2 ).toInt();
            QString   number   = QString::number( counter );
            if( number.size() < fld_size ) {
               number = QString( fld_size - number.size(), '0' ) + number;
            }
            tmp.replace( tags[i], number );
            ++counter;
         }
      }
      fname = tmp;
   }

   QString dst_fpath = dst_dir;
   if( !dst_fpath.endsWith( QDir::separator() ) ) dst_fpath += QDir::separator();
   dst_fpath += fname;
   dst_fpath += '.';
   dst_fpath += fext;

   return dst_fpath;
}
// end of dst_fpath

//*******************************************************************
// rename_muster_valid                                       PRIVATE
//*******************************************************************
bool QBtProcessDialog::rename_muster_valid() const
{
   const QString text = rename_muster_->text();
   if( text.isEmpty() ) return false;
   if( text.contains( QDir::separator() ) ) return false;

   const QStringList data = tags( text );
   const int n = data.size();
   if(( n < 1 ) || ( n > 2 )) return false;

   if( n == 2 ) {
      if( data[0].endsWith( "f" ) && data[1].endsWith( "f" )) return false;
      if( data[0].endsWith( "d" ) && data[1].endsWith( "d" )) return false;
   }
   
   return true;
}
// end of rename_muster_valid

//*******************************************************************
// tags                                                      PRIVATE
//-------------------------------------------------------------------
// Wyszukiwanie wszystkich tagow w przyslanym wzorcu.
// Szukamy tagow: %f oraz %d*d (jeden lub wiecej cyfr).
//*******************************************************************
QStringList QBtProcessDialog::tags( const QString& text ) const
{
   static const QRegExp rx( "%(\\d*d|f)" );

   QStringList tags = QStringList();
   //............................................
   int idx = rx.indexIn( text );
   while( idx != -1 ) {
      const int n = rx.matchedLength();
      tags.push_back( text.mid( idx, n ) );
      idx = rx.indexIn( text, idx + n );
   }
   //............................................
   return tags;
}
// end of tags
