/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtView.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 19.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtView.h"
#include "QBtViewer.h"
#include "QBtInfo.h"
#include "QBtEventsController.h"
#include "QBtConfigWatermark.h"
#include "QBtConfigRotate.h"
#include "QBtConfigResize.h"
#include "QBtConfigBorder.h"
#include "QBtConfigAnnotate.h"
#include "QBtConfigGrayScale.h"
#include "QBtConfigNegate.h"
#include "QBtConfigNormalize.h"
#include "QBtConfigEnhance.h"
#include "QBtConfigEqualize.h"
#include <QDir>
#include <QFile>
#include <QDateTime>
#include <QKeyEvent>
#include <QScrollBar>
#include <QApplication>
#include <QInputDialog>
#include <QMessageBox>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtView::Name         = QT_TR_NOOP( "Name" );
const char* const QBtView::Ext          = QT_TR_NOOP( "Ext" );
const char* const QBtView::Access       = QT_TR_NOOP( "Access" );
const char* const QBtView::Size         = QT_TR_NOOP( "Size" );
const char* const QBtView::Date         = QT_TR_NOOP( "Date" );
const char* const QBtView::Time         = QT_TR_NOOP( "Time" );
const char* const QBtView::MkDirCaption = QT_TR_NOOP( "New directory" );
const char* const QBtView::MkDirPrompt  = QT_TR_NOOP( "Enter a directory name:" );
const char* const QBtView::Error        = QT_TR_NOOP( "Error" );
const char* const QBtView::DirNotEmpty  = QT_TR_NOOP( "Can't delete the directory: %1\n"
                                                      "The directory propably is not empty!" );

//*******************************************************************
// QBtView                                               CONSTRUCTOR
//*******************************************************************
QBtView::QBtView( const QString& in_path, QWidget* const in_parent )
: QTreeWidget( in_parent )
, path_      ( in_path )
, dir_       ()
, names_     ( NamesStack() )
, exts_      ( Extensions() )
{
   exts_.insert( "gif" );
   exts_.insert( "jpg" );
   exts_.insert( "jpeg" );
   exts_.insert( "png" );
   exts_.insert( "tiff" );
   exts_.insert( "bmp" );

   setRootIsDecorated( false );
   setAllColumnsShowFocus( false );
   setEditTriggers( NoEditTriggers );
   setSelectionMode( SingleSelection );
   setSelectionBehavior( SelectRows );
   setUniformRowHeights( true );
   setVerticalScrollMode( ScrollPerItem );

   QTreeWidgetItem* const hdr = new QTreeWidgetItem;
   hdr->setText( FLD_CHECK_INDEX , QString() );
   hdr->setText( FLD_NAME_INDEX  , tr( Name ) );
   hdr->setText( FLD_EXT_INDEX   , tr( Ext ) );
   hdr->setText( FLD_ACCESS_INDEX, tr( Access ) );
   hdr->setText( FLD_SIZE_INDEX  , tr( Size ) );
   hdr->setText( FLD_DATE_INDEX  , tr( Date ) );
   hdr->setText( FLD_TIME_INDEX  , tr( Time ) );

   hdr->setTextAlignment( FLD_NAME_INDEX  , Qt::AlignHCenter );
   hdr->setTextAlignment( FLD_EXT_INDEX   , Qt::AlignHCenter );
   hdr->setTextAlignment( FLD_ACCESS_INDEX, Qt::AlignHCenter );
   hdr->setTextAlignment( FLD_SIZE_INDEX  , Qt::AlignHCenter );
   hdr->setTextAlignment( FLD_DATE_INDEX  , Qt::AlignHCenter );

   setHeaderItem( hdr );

   connect( this, SIGNAL( itemDoubleClicked(QTreeWidgetItem*,int) ), this, SLOT( execute() ) );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::F4 );
   evc->append( this, QBtEvent::F7 );
   evc->append( this, QBtEvent::F8 );
   evc->append( this, QBtEvent::GrayscaleRequest );
   evc->append( this, QBtEvent::NegateRequest );
   evc->append( this, QBtEvent::NormalizeRequest );
   evc->append( this, QBtEvent::EnhanceRequest );
   evc->append( this, QBtEvent::EqualizeRequest ); 
   evc->append( this, QBtEvent::WatermarkRequest );
   evc->append( this, QBtEvent::RotateRequest );
   evc->append( this, QBtEvent::ResizeRequest );
   evc->append( this, QBtEvent::BorderRequest );
   evc->append( this, QBtEvent::AnnotateRequest );
}
// end of QBtView

//*******************************************************************
// ~QBtView                                               DESTRUCTOR
//*******************************************************************
QBtView::~QBtView()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtView

//*******************************************************************
// focusInEvent                                    PRIVATE inherited
//*******************************************************************
void QBtView::focusInEvent( QFocusEvent* const in_event )
{
   QBtEventsController::instance()->send_event( QBtEvent::DirChanged, path_ );
   QTreeWidget::focusInEvent( in_event );
}
// end of focusInEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtView::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::F4:
         info();
         break;
      case QBtEvent::F7:
         make_dir();
         break;
      case QBtEvent::F8:
         remove();
         break;
      case QBtEvent::GrayscaleRequest:
         grayscale();
         break;
      case QBtEvent::NegateRequest:
         negate();
         break;
      case QBtEvent::NormalizeRequest:
         normalize();
         break;
      case QBtEvent::EnhanceRequest:
         enhance();
         break;
      case QBtEvent::EqualizeRequest:
         equalize();
         break;
      case QBtEvent::WatermarkRequest:
         watermark();
         break;
      case QBtEvent::RotateRequest:
         rotate();
         break;
      case QBtEvent::ResizeRequest:
         resize();
         break;
      case QBtEvent::BorderRequest:
         border();
         break;
      case QBtEvent::AnnotateRequest:
         annotate();
         break;
   }
}
// end of customEvent

//*******************************************************************
// read                                                      PRIVATE
//*******************************************************************
bool QBtView::read( const QString& in_path_ )
{
   if( !dir_.cd( in_path_ ) ) return false;
   read();
   goto_top();
   return true;
}
void QBtView::read()
{
   clear();
   path_ = dir_.absolutePath();
   QBtEventsController::instance()->send_event( QBtEvent::DirChanged, path_ );
   
   static const QDir::Filters what   = QDir::AllDirs
                                     | QDir::NoDotAndDotDot
                                     | QDir::Readable
                                     | QDir::Files
                                     | QDir::NoSymLinks;
   static const QDir::SortFlags sort = QDir::Name
                                     | QDir::DirsFirst
                                     | QDir::IgnoreCase
                                     | QDir::LocaleAware;

   if( !dir_.isRoot() ) add_dot_dot();
   
   dir_.setFilter( what );
   dir_.setSorting( sort );

   const QFileInfoList files = dir_.entryInfoList();
   QFileInfoList::const_iterator it = files.begin();
   const QFileInfoList::const_iterator end = files.end();
   while( it != end ) {
      add_item( *it );
      ++it;
   }
   //......................................................
   if( invisibleRootItem()->childCount() ) {
      resize_columns();
   }
}
// end of read

void QBtView::reread() 
{
   disable_updates();
   read();
   enable_updates();      
   goto_top();
}

//*******************************************************************
// add_dot_dot                                               PRIVATE
//*******************************************************************
void QBtView::add_dot_dot()
{
   static const QString DotDot = "..";
   static const QIcon   DirIcon    = QIcon( ":/img/folder.png" );
   
   QTreeWidgetItem* const item = new QTreeWidgetItem;
   item->setText( FLD_NAME_INDEX, DotDot );
   item->setIcon( FLD_NAME_INDEX, DirIcon );
   item->setData( FLD_NAME_INDEX, IS_DIR, true );
   item->setData( FLD_NAME_INDEX, IS_PIC, false );
   invisibleRootItem()->addChild( item );
}
// end of add_dot_dot

//*******************************************************************
// add_item                                                  PRIVATE
//-------------------------------------------------------------------
// Utworzenie informacji o wskazanym pliku i dodanie go do widoku.
//*******************************************************************
void QBtView::add_item( const QFileInfo& in_file )
{
   static const QString Fld        = " %1 ";
   static const QString DirMark    = "<DIR>";
   static const QString DateMuster = "yyyy-MM-dd";
   static const QString TimeMuster = "hh-mm-ss";
   static const QIcon   DirIcon    = QIcon( ":/img/folder.png" );
   static const QIcon   NoReadIcon = QIcon( ":/img/noread.png" );
   
   QTreeWidgetItem* const item = new QTreeWidgetItem;
   item->setText( FLD_NAME_INDEX  , in_file.completeBaseName() );
   item->setText( FLD_EXT_INDEX   , in_file.suffix() );
   item->setText( FLD_ACCESS_INDEX, access( in_file ) );
   item->setText( FLD_DATE_INDEX  , Fld.arg( in_file.lastModified().date().toString( DateMuster ) ) );
   item->setText( FLD_TIME_INDEX  , Fld.arg( in_file.lastModified().time().toString( TimeMuster ) ) );

   
   item->setTextAlignment( FLD_DATE_INDEX, Qt::AlignHCenter );

   if( in_file.isDir() ) {
      item->setIcon( FLD_NAME_INDEX, in_file.isReadable() ? DirIcon : NoReadIcon);
      item->setText( FLD_SIZE_INDEX, DirMark );
      item->setTextAlignment( FLD_SIZE_INDEX, Qt::AlignHCenter );
      //.........................................
      item->setData( FLD_NAME_INDEX, IS_DIR, true );
      item->setData( FLD_NAME_INDEX, IS_PIC, false );
   }
   else {
      item->setText( FLD_SIZE_INDEX, QBtShared::num2str( in_file.size() ) );
      item->setTextAlignment( FLD_SIZE_INDEX, Qt::AlignRight );
      //.........................................
      item->setData( FLD_NAME_INDEX, IS_DIR, false );
      if( exts_.find( in_file.suffix().toLower() ) != exts_.end() ) {
         item->setCheckState( FLD_CHECK_INDEX, Qt::Unchecked );
         item->setData( FLD_NAME_INDEX, IS_PIC, true );
      }
      else {
         item->setData( FLD_NAME_INDEX, IS_PIC, false );
      }
   }
   item->setData( FLD_NAME_INDEX, PATH, in_file.absoluteFilePath() );

   invisibleRootItem()->addChild( item );
}
// end of add_item

//*******************************************************************
// resize_columns                                            PRIVATE
//*******************************************************************
void QBtView::resize_columns()
{
   const int n = columnCount() - 1; // ostatniej kolumny nie stroimy
   for( int col = 0; col < n; ++col ) {
      resizeColumnToContents( col );
   }
}
// end of resize_columns

//*******************************************************************
// access
//*******************************************************************
QString QBtView::access( const QFileInfo& in_fi ) const
{
   static const QChar DirMark   = 'd';
   static const QChar LinkMark  = 'l';
   static const QChar ReadMark  = 'r';
   static const QChar WriteMark = 'w';
   static const QChar ExeMark   = 'x';
   
   const QFile::Permissions p = in_fi.permissions();
   QString buffer = "----------";

        if( in_fi.isDir()     ) buffer[ 0 ] = DirMark;
   else if( in_fi.isSymLink() ) buffer[ 0 ] = LinkMark;
   
   if( p & QFile::ReadUser   ) buffer[ 1 ] = ReadMark;
   if( p & QFile::WriteUser  ) buffer[ 2 ] = WriteMark;
   if( p & QFile::ExeUser    ) buffer[ 3 ] = ExeMark;
   if( p & QFile::ReadGroup  ) buffer[ 4 ] = ReadMark;
   if( p & QFile::WriteGroup ) buffer[ 5 ] = WriteMark;
   if( p & QFile::ExeGroup   ) buffer[ 6 ] = ExeMark;
   if( p & QFile::ReadOther  ) buffer[ 7 ] = ReadMark;
   if( p & QFile::WriteOther ) buffer[ 8 ] = WriteMark;
   if( p & QFile::ExeOther   ) buffer[ 9 ] = ExeMark;

   return buffer;
}
// access

//*******************************************************************
// goto_top                                                  PRIVATE
//*******************************************************************
void QBtView::goto_top()
{
   QTreeWidgetItem* const item = invisibleRootItem()->child( 0 );
   if( item ) {
      setCurrentItem( item );
      scrollToItem( item, PositionAtTop );
   }
   update();
}
// end of goto_top

//*******************************************************************
// goto_item                                                 PRIVATE
//*******************************************************************
void QBtView::goto_item( QTreeWidgetItem* const in_item )
{
   setCurrentItem( in_item );
   scrollToItem( in_item );
}
// end of goto_item

//*******************************************************************
// goto_end                                                  PRIVATE
//*******************************************************************
void QBtView::goto_end()
{
   const QTreeWidgetItem* const root = invisibleRootItem();
   int n = root->childCount();

   if( n > 0 ) {
      --n;
      QTreeWidgetItem* const item = root->child( n );
      if( item ) {
         setCurrentItem( item );
         scrollToItem( item, PositionAtBottom );
      }
      update();
   }
}
// end of goto_end

//*******************************************************************
// drawRow                                         PRIVATE inherited
//*******************************************************************
void QBtView::drawRow( QPainter*                   in_painter,
                       const QStyleOptionViewItem& in_style  ,
                       const QModelIndex&          in_index  ) const
{
   static const QColor   FOCUS_CURR_BACKGROUND = qRgb( 130,170,170 ); // QColor( 150, 150, 200 )
   static const QColor   FOCUS_CURR_FOREGROUND = qRgb( 255,255,255 );
   static const QColor NOFOCUS_CURR_BACKGROUND = qRgb( 170,170,170 );
   static const QColor NOFOCUS_CURR_FOREGROUND = qRgb( 255,255,255 );
   static const QColor NOFOCUS_NORM_FOREGROUND = qRgb( 150,150,150 );
   
   const QPalette this_palette = in_style.palette;
   
   if( hasFocus() ) {
      static bool     is_valid    = false;
      static QPalette org_palette = QPalette();
      static QPalette my_palette  = QPalette();
             bool     set_palette = false;
      
      if( is_valid ) {
         set_palette = ( org_palette != this_palette );
      }
      else {
         set_palette = is_valid = true;
      }
      if( set_palette ) {
         my_palette = org_palette = this_palette;
         my_palette.setColor( QPalette::Highlight      , FOCUS_CURR_BACKGROUND );
         my_palette.setColor( QPalette::HighlightedText, FOCUS_CURR_FOREGROUND );
      }
      
      QStyleOptionViewItem style = in_style;
      style.palette = my_palette;
      QTreeWidget::drawRow( in_painter, style, in_index );
      
   }
   else {
      static bool     is_valid    = false;
      static QPalette org_palette = QPalette();
      static QPalette my_palette  = QPalette();
             bool     set_palette = false;

      if( is_valid ) {
         set_palette = ( org_palette != this_palette );
      }
      else {
         set_palette = is_valid = true;
      }
      if( set_palette ) {
         my_palette = org_palette = this_palette;
         my_palette.setColor( QPalette::Highlight      , NOFOCUS_CURR_BACKGROUND );
         my_palette.setColor( QPalette::HighlightedText, NOFOCUS_CURR_FOREGROUND );
         my_palette.setColor( QPalette::Text           , NOFOCUS_NORM_FOREGROUND );
      }
      QStyleOptionViewItem style = in_style;
      style.palette = my_palette;
      QTreeWidget::drawRow( in_painter, style, in_index );      
   }
}
// end of drawRow

//*******************************************************************
// keyPressEvent                                   PRIVATE inherited
//*******************************************************************
void QBtView::keyPressEvent( QKeyEvent* const in_event )
{
   if( Qt::ControlModifier == in_event->modifiers() ) {
      switch( in_event->key() ) {
         case Qt::Key_R:
            disable_updates();
            read();
            goto_top();
            enable_updates();
            break;
         case Qt::Key_A:
            select_pictures();
            break;
      }
   }
   else {
      switch( in_event->key() ) {
         case Qt::Key_Return:
         case Qt::Key_Enter:
            execute();
            break;
         case Qt::Key_Home:
            goto_top();
            break;
         case Qt::Key_End:
            goto_end();
            break;
         case Qt::Key_Backspace:
            cd_up();
            break;
         case Qt::Key_F1:
            QBtEventsController::instance()->send_event( QBtEvent::F1 );
            break;
         case Qt::Key_F3:
            viewer();
            break;
         case Qt::Key_F4:
            info();
            break;
         case Qt::Key_F5:
            QBtEventsController::instance()->send_event( QBtEvent::F5 );
            break;
         case Qt::Key_F7:
            make_dir();
            break;
         case Qt::Key_F8:
         case Qt::Key_Delete:
            remove();
            break;
         case Qt::Key_F10:
            QBtEventsController::instance()->send_event( QBtEvent::F10 );
            break;
         default:
            QTreeWidget::keyPressEvent( in_event );
      }
   }
}
// end of keyPressEvent

//*******************************************************************
// execute                                                   PRIVATE
//*******************************************************************
void QBtView::execute()
{
   static const QString Dot = ".";
   static const QString DotDot = "..";
   
   const QTreeWidgetItem* const item = currentItem();
   if( !item ) return;

   if( item->data( FLD_NAME_INDEX, IS_DIR ).toBool() ) {
      if( DotDot == item->text( FLD_NAME_INDEX ) ) {
         cd_up();
      }
      else {
         QString fname = item->text( FLD_NAME_INDEX );
         const QString fext  = item->text( FLD_EXT_INDEX );
         if( !fext.isEmpty() ) {
            fname += Dot;
            fname += fext;
         }
         if( dir_.cd( fname ) ) {
            disable_updates();
            names_.push( item->data( FLD_NAME_INDEX, PATH ).toString() );
            read();
            goto_top();
            enable_updates();
         }
      }
   }
}
// end of execute

//*******************************************************************
// cd_up                                                     PRIVATE
//*******************************************************************
void QBtView::cd_up()
{
   if( dir_.cdUp() ) {
      disable_updates();
      read();
      //.........................................
      if( names_.empty() ) {
         goto_top();
      }
      else {
         QTreeWidgetItem* const item = search_item( names_.top() );
         if( item ) {
            names_.pop();
            goto_item( item );
         }
         else {
            names_ = NamesStack();
            goto_top();
         }
      }
      //.........................................
      enable_updates();
   }
}
// end of cd_up

//*******************************************************************
// delete_file                                               PRIVATE
//-------------------------------------------------------------------
// UWAGA: Na uzytek Viewera.
//*******************************************************************
void QBtView::delete_file( const QString& fpath )
{
   QTreeWidgetItem* const item = search_file( fpath );
   if( item ) invisibleRootItem()->removeChild( item );
}
// end of delete_file

//*******************************************************************
// current_file                                              PRIVATE
//-------------------------------------------------------------------
// UWAGA: Na uzytek Viewera
//*******************************************************************
void QBtView::current_file( const QString& fpath )
{
   QTreeWidgetItem* const item = search_file( fpath );
   if( item ) goto_item( item );
}
// end of current_file

//*******************************************************************
// check_file                                                 PUBLIC
//*******************************************************************
void QBtView::check_file( const QString& fpath )
{
   QTreeWidgetItem* const item = search_file( fpath );
   
   if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
      if( Qt::Checked == item->checkState( FLD_CHECK_INDEX ) ) {
         item->setCheckState( FLD_CHECK_INDEX, Qt::Unchecked );
      }
      else {
         item->setCheckState( FLD_CHECK_INDEX, Qt::Checked );
      }
   }
}
// end of check_file

//*******************************************************************
// search_file                                               PRIVATE
//*******************************************************************
QTreeWidgetItem* QBtView::search_file( const QString& fpath )
{
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      QTreeWidgetItem* item = *it;
      if( fpath == item->data( FLD_NAME_INDEX, PATH ).toString() ) {
         return item;
      }
      ++it;
   }
   return 0;
}
// end of search_file

//*******************************************************************
// search_item                                               PRIVATE
//-------------------------------------------------------------------
// Wyszukiwanie wsrod aktualnie wyswietlanych katalogo takiego,
// ktore sciezka odpowiada tej ktora przyslano.
//*******************************************************************
QTreeWidgetItem* QBtView::search_item( const QString& in_fpath )
{
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      QTreeWidgetItem* const item = *it;
      if( item->data( FLD_NAME_INDEX, IS_DIR ).toBool() ) {
         if( in_fpath == item->data( FLD_NAME_INDEX, PATH ).toString() ) {
            return item;
         }
      }
      ++it;
   }
   return 0;
}
QTreeWidgetItem* QBtView::search_item( const QString& in_fname, const QString& in_fext )
{
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      QTreeWidgetItem* const item = *it;
      if( in_fname == item->text( FLD_NAME_INDEX ) ) {
         if( in_fext == item->text( FLD_EXT_INDEX ) ) {
            return item;
         }
      }
      ++it;
   }
   return 0;
}
// end of search_item

//*******************************************************************
// search_pics                                               PRIVATE
//-------------------------------------------------------------------
// Wyszukiwanie wszystkich(!) plików obrazów.
//*******************************************************************
Strings QBtView::search_pics()
{
   Strings data = Strings();
   
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      const QTreeWidgetItem* const item = *it;
      if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
         data.push_back( item->data( FLD_NAME_INDEX, PATH ).toString() );
      }
      ++it;
   }
   return data;
}
// end of search_pics

//*******************************************************************
// select_pictures                                           PRIVATE
//*******************************************************************
void QBtView::select_pictures()
{
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      QTreeWidgetItem* const item = *it;
      if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
         if( Qt::Checked == item->checkState( FLD_CHECK_INDEX ) ) {
            item->setCheckState( FLD_CHECK_INDEX, Qt::Unchecked );
         }
         else {
            item->setCheckState( FLD_CHECK_INDEX, Qt::Checked );
         }
      }
      ++it;
   }
}
// end of select_pictures

//*******************************************************************
// selected_items                                            PRIVATE
//*******************************************************************
QBtView::SelectedItems QBtView::selected_items()
{
   SelectedItems retval = SelectedItems();
   
   QTreeWidgetItemIterator it( this );
   while( *it ) {
      QTreeWidgetItem* const item = *it;
      if( Qt::Checked == item->checkState( FLD_CHECK_INDEX ) ) {
         retval.push_back( item );
      }
      ++it;
   }
   return retval;
   
}
// end of SelectedItems

//*******************************************************************
// get_selected                                               PUBLIC
//-------------------------------------------------------------------
// Zwraca informacje o wyselekcjonowanych pozycjach.
// Informacja ta to pelna sciezka do pliku.
//*******************************************************************
Strings QBtView::get_selected()
{
   Strings data = Strings();

   QTreeWidgetItemIterator it( this );
   while( *it ) {
      const QTreeWidgetItem* const item = *it;
      if( Qt::Checked == item->checkState( FLD_CHECK_INDEX ) ) {
         data.push_back( item->data( FLD_NAME_INDEX, PATH ).toString() );
      }
      ++it;
   }   
   return data;
}
// end of get_selected

//*******************************************************************
// get_selected_set                                          PRIVATE
//*******************************************************************
StringSet QBtView::get_selected_set()
{
   StringSet data = StringSet();

   QTreeWidgetItemIterator it( this );
   while( *it ) {
      const QTreeWidgetItem* const item = *it;
      if( Qt::Checked == item->checkState( FLD_CHECK_INDEX ) ) {
         data.insert( item->data( FLD_NAME_INDEX, PATH ).toString() );
      }
      ++it;
   }   
   return data;
}
// end of get_selected_set

//*******************************************************************
// get_current                                                PUBLIC
//-------------------------------------------------------------------
// Zwraca pelna sciezke do pliku reprezentowanego przez aktualny
// element.
//*******************************************************************
QString QBtView::get_current() const
{
   const QTreeWidgetItem* const item = currentItem();
   if( item ) {
      return item->data( FLD_NAME_INDEX, PATH ).toString();
   }
   return QString();
}
// end of get_current

//*******************************************************************
// disable_updates                                           PRIVATE
//*******************************************************************
void QBtView::disable_updates()
{
   QBtShared::set_cursor( Qt::WaitCursor );
   setUpdatesEnabled( false );
   qApp->processEvents();
}
// end of disable_updates

//*******************************************************************
// enable_updates                                            PRIVATE
//*******************************************************************
void QBtView::enable_updates()
{
   setUpdatesEnabled( true );
   QBtShared::restore_cursor();
   qApp->processEvents();
}
// end of enable_updates

//*******************************************************************
// viewer                                                     PUBLIC
//*******************************************************************
void QBtView::viewer()
{
   Strings files = search_pics();
   if( files.empty() ) return;

   int index = int();
   StringSet checked = get_selected_set();
   
   const QTreeWidgetItem* const item = currentItem();
   if( item ) {
      if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
         const QString current = item->data( FLD_NAME_INDEX, PATH ).toString();
         const int n = files.size();
         for( int i = 0; i < n; ++i ) {
            if( current == files[ i ] ) {
               index = i;
               break;
            }
         }
      }
   }
   QBtViewer viewer( this, files, index, checked );
   viewer.exec();
}
// end of viewer

//*******************************************************************
// make_dir                                                  PRIVATE
//*******************************************************************
void QBtView::make_dir()
{
   if( !hasFocus() ) return;
   //......................................................
   bool ok;
   const QString name = QInputDialog::getText( this,
                                               tr( MkDirCaption ),
                                               tr( MkDirPrompt ),
                                               QLineEdit::Normal,
                                               QString(),
                                               &ok );
   if( !ok || name.isEmpty() ) return;
   //......................................................
   if( dir_.mkdir( name ) ) {
      disable_updates();
      read();
      enable_updates();      

      const QFileInfo fi( name );
      QTreeWidgetItem* const item = search_item( fi.completeBaseName(), fi.suffix() );
      if( item ) goto_item( item );
   }
}
// end of make_dir

//*******************************************************************
// remove                                                    PRIVATE
//*******************************************************************
void QBtView::remove()
{
   if( !hasFocus() ) return;
   
   QTreeWidgetItem* const item = currentItem();
   if( !item ) return;
   QTreeWidgetItem* next_item = item;
   
   const SelectedItems selections = selected_items();
   if( selections.empty() ) {
      // W sytuacji gdzy uzytkownik nie zaznaczyl zadnych obrazkow
      // usuwamy aktualna pozycje.
      next_item = get_next_item( item );
      remove_item( item );
   }
   else {
      // Jesli sa zaznaczone obrazki to usuwamy tylko to
      // co zaznaczyl uzytkownik.
      SelectedItems::const_iterator it = selections.begin();
      const SelectedItems::const_iterator end = selections.end();
      while( it != end ) {
         QTreeWidgetItem* const current = *it;
         if( current == item ) next_item = get_next_item( current );
         remove_item( current );
         ++it;
      }
   }
   goto_item( next_item );
}
// end of remove

//*******************************************************************
// remove_item                                               PRIVATE
//*******************************************************************
void QBtView::remove_item( QTreeWidgetItem* const in_item )
{
   static const QString DotDot = "..";
   if( !in_item ) return;
   
   const QString fname = get_full_name( in_item );
   const bool is_dir = in_item->data( FLD_NAME_INDEX, IS_DIR ).toBool();
   if( DotDot == fname ) return;
   
   const bool is_ok = is_dir ? dir_.rmdir( fname ) : dir_.remove( fname );
   
   if( is_ok ) {
      invisibleRootItem()->removeChild( in_item );
   }
   else {
      if( is_dir ) {
         QMessageBox::warning( this, tr( Error ), tr( DirNotEmpty ).arg( fname ) );
      }
   }
}
// end of remove_item

//*******************************************************************
// get_next_item                                             PRIVATE
//*******************************************************************
QTreeWidgetItem* QBtView::get_next_item( const QTreeWidgetItem* const in_item ) const
{
   QTreeWidgetItem* const retval = itemBelow( in_item );
   if( retval ) return retval;
   return itemAbove( in_item );   
}
// end of get_next_item

//*******************************************************************
// get_full_name                                             PRIVATE
//*******************************************************************
QString QBtView::get_full_name( const QTreeWidgetItem* const in_item ) const
{
   if( !in_item ) return QString();

   QString fname = in_item->text( FLD_NAME_INDEX );
   const QString fext = in_item->text( FLD_EXT_INDEX );
   if( !fext.isEmpty() ) {
      fname += '.';
      fname += fext;
   }
   return fname;
}
// end of get_full_name

//*******************************************************************
// info                                                      PRIVATE
//-------------------------------------------------------------------
// Startuje proces odczytu informacji (metadata) o obrazie.
//*******************************************************************
void QBtView::info()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtInfo dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            dialog.exec();
         }
      }
   }
}
// end of info

//*******************************************************************
// grayscale                                                 PRIVATE
//*******************************************************************
void QBtView::grayscale()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigGrayScale dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of grayscale

//*******************************************************************
// negate                                                    PRIVATE
//*******************************************************************
void QBtView::negate()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigNegate dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of negate

//*******************************************************************
// normalize                                                 PRIVATE
//*******************************************************************
void QBtView::normalize()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigNormalize dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of normalize

//*******************************************************************
// enhance                                                   PRIVATE
//*******************************************************************
void QBtView::enhance()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigEnhance dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of emhance

//*******************************************************************
// equalize                                                  PRIVATE
//*******************************************************************
void QBtView::equalize()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigEqualize dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of equalize

//*******************************************************************
// watermark                                                 PRIVATE
//*******************************************************************
void QBtView::watermark()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigWatermark dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of annotate

//*******************************************************************
// rotate                                                    PRIVATE
//*******************************************************************
void QBtView::rotate()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigRotate dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of rotate

//*******************************************************************
// resize                                                    PRIVATE
//*******************************************************************
void QBtView::resize()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigResize dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of resize

//*******************************************************************
// border                                                    PRIVATE
//*******************************************************************
void QBtView::border()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigBorder dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of border

//*******************************************************************
// annotate                                                  PRIVATE
//*******************************************************************
void QBtView::annotate()
{
   if( hasFocus() ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( item->data( FLD_NAME_INDEX, IS_PIC ).toBool() ) {
            QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
            QBtConfigAnnotate dialog( this, item->data( FLD_NAME_INDEX, PATH ).toString() );
            if( QDialog::Rejected == dialog.exec() ) {
               QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
            }
         }
      }
   }
}
// end of annotate
