/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtViewer.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 29.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtViewer.h"
#include "QBtShared.h"
#include "QBtMagic.h"
#include "QBtPicture.h"
#include "QBtView.h"
#include "QBtEventsController.h"
#include <QScrollBar>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QLabel>
#include <QIcon>
#include <QDesktopWidget>
#include <QApplication>
#include <QCheckBox>
#include <QMessageBox>
//#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtViewer::Caption     = QT_TR_NOOP( "Viewer" );
const char* const QBtViewer::Save        = QT_TR_NOOP( "save changed image" );
const char* const QBtViewer::Close       = QT_TR_NOOP( "&Close" );
const char* const QBtViewer::Scale       = QT_TR_NOOP( "&scalling" );
const char* const QBtViewer::Error       = QT_TR_NOOP( "Error on reading the picture:\n%1" );
const char* const QBtViewer::Previous    = QT_TR_NOOP( "previous image" );
const char* const QBtViewer::Next        = QT_TR_NOOP( "next image" );
const char* const QBtViewer::RotateLeft  = QT_TR_NOOP( "rotate left" );
const char* const QBtViewer::RotateRight = QT_TR_NOOP( "rotate right" );
const char* const QBtViewer::Check       = QT_TR_NOOP( "select image" );
const char* const QBtViewer::Delete      = QT_TR_NOOP( "delete image" );
const char* const QBtViewer::DeleteError = QT_TR_NOOP( "Can't delete the file:\n%1" );


//*******************************************************************
// QBtViewer                                             CONSTRUCTOR
//*******************************************************************
QBtViewer::QBtViewer( QWidget* const parent, const Strings& files, const int index, const StringSet& checked )
: QDialog  ( parent )
, files_   ( files )
, index_   ( index )
, checked_ ( checked )
, panel_   ( new QWidget )
, picture_ ( new QBtPicture )
, scale_   ( new QCheckBox( tr( Scale ) ) )
, left_    ( new QPushButton( QIcon( ":/img/to-left.png" ), QString() ) )
, right_   ( new QPushButton( QIcon( ":/img/to-right.png" ), QString() ) )
, prev_    ( new QPushButton( QIcon( ":/img/prev.png" ), QString() ) )
, next_    ( new QPushButton( QIcon( ":/img/next.png" ), QString() ) )
, check_   ( new QPushButton( QIcon( ":/img/checkbox.png" ), QString() ) )
, delete_  ( new QPushButton( QIcon( ":/img/trash.png" ), QString() ) )
, save_    ( new QPushButton( QIcon( ":/img/floppy.png" ), QString() ) )
, close_   ( new QPushButton( tr( Close ) ) )
, image_   ()
, angle_   ( 0 )
, ok_      ( false )
, scaled_  ( true )
{
   const QSize sic = prev_->iconSize();
   const int n = prev_->iconSize().width() + 4;
   prev_->setMaximumWidth( n );
   prev_->setMaximumHeight( n );
   next_->setMaximumWidth( n );
   next_->setMaximumHeight( n );
   left_->setMaximumWidth( n );
   left_->setMaximumHeight( n );
   right_->setMaximumWidth( n );
   right_->setMaximumHeight( n );
   close_->setDefault( true );

   prev_->setToolTip( tr( Previous ) );
   next_->setToolTip( tr( Next ) );
   check_->setToolTip( tr( Check ) );
   delete_->setToolTip( tr( Delete ) );
   save_->setToolTip( tr( Save ) );

   check_->setCheckable( true );

   scale_->setCheckState( Qt::Checked );
   QHBoxLayout* const btn_layout = new QHBoxLayout;
   btn_layout->addWidget( scale_ );
   btn_layout->addStretch();
   btn_layout->addWidget( left_ );
   btn_layout->addWidget( right_ );
   btn_layout->addStretch();
   btn_layout->addWidget( prev_ );
   btn_layout->addWidget( next_ );
   btn_layout->addStretch();
   btn_layout->addWidget( check_ );
   btn_layout->addWidget( delete_ );
   btn_layout->addWidget( save_ );
   btn_layout->addWidget( close_ );

   QVBoxLayout* const img_layout = new QVBoxLayout;
   img_layout->setContentsMargins( 0, 0, 0, 0 );
   img_layout->addWidget( picture_ );
   panel_->setLayout( img_layout );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addWidget( panel_ );
   main_layout->addLayout( btn_layout );
   setLayout( main_layout );

   connect( scale_ , SIGNAL( stateChanged(int) ), this, SLOT( scale_changed(int) ) );
   connect( close_ , SIGNAL( clicked() ), this, SLOT( accept() ) );
   connect( prev_  , SIGNAL( clicked() ), this, SLOT( prev() ) );
   connect( next_  , SIGNAL( clicked() ), this, SLOT( next() ) );
   connect( left_  , SIGNAL( clicked() ), this, SLOT( left() ) );
   connect( right_ , SIGNAL( clicked() ), this, SLOT( right() ) );
   connect( save_  , SIGNAL( clicked() ), this, SLOT( save_clicked() ) );
   connect( check_ , SIGNAL( clicked() ), this, SLOT( check_clicked() ) );
   connect( delete_, SIGNAL( clicked() ), this, SLOT( delete_clicked() ) );
}
QBtViewer::QBtViewer( QWidget* const parent, const QImage& image )
: QDialog  ( parent )
, files_   ( Strings() )
, index_   ( 0 )
, panel_   ( new QWidget )
, picture_ ( new QBtPicture )
, scale_   ( new QCheckBox( tr( Scale ) ) )
, left_    ( 0 )
, right_   ( 0 )
, prev_    ( 0 )
, next_    ( 0 )
, check_   ( 0 )
, delete_  ( 0 )
, save_    ( 0 )
, close_   ( new QPushButton( tr( Close ) ) )
, image_   ( image )
, angle_   ( 0 )
, ok_      ( true )
, scaled_  ( true )
{
   setWindowTitle( tr( Caption ) );
   close_->setDefault( true );

   scale_->setCheckState( Qt::Checked );
   //......................................................
   QHBoxLayout* const btn_layout = new QHBoxLayout;
   btn_layout->addWidget( scale_ );
   btn_layout->addStretch();
   btn_layout->addWidget( close_ );
   //......................................................
   QVBoxLayout* const img_layout = new QVBoxLayout;
   img_layout->setContentsMargins( 0, 0, 0, 0 );
   img_layout->addWidget( picture_ );
   panel_->setLayout( img_layout );
   //......................................................
   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addWidget( panel_ );
   main_layout->addLayout( btn_layout );
   setLayout( main_layout );

   connect( scale_, SIGNAL(stateChanged(int)), this, SLOT(scale_changed(int)));
   connect( close_, SIGNAL(clicked()), this, SLOT(accept()));
}
// end of QBtViewer

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtViewer::showEvent( QShowEvent* const in_event )
{
   QDialog::showEvent( in_event );

   const int   screen_width   = qApp->desktop()->availableGeometry().width();
   const int   screen_height  = qApp->desktop()->availableGeometry().height();
   const qreal width_percent  = 60.0 / 100.0;
   const qreal height_percent = 60.0 / 100.0;    
   const int   dx = static_cast<int>( screen_width  * width_percent );
   const int   dy = static_cast<int>( screen_height * height_percent );
   const int   x  = ( screen_width  - dx + 1 ) >> 1;
   const int   y  = ( screen_height - dy + 1 ) >> 1;
   move( x, y );
   resize( dx, dy );

   if( files_.size() ) {
      display();
   }
   else {
      resize_image();
   }
}
// end of ~QBtViewer

//*******************************************************************
// resizeEvent                                     PRIVATE inherited
//*******************************************************************
void QBtViewer::resizeEvent( QResizeEvent* const e )
{
   QDialog::resizeEvent( e );
   if( !ok_ ) return;
   resize_image();
}
// end of resizeEvent

//*******************************************************************
// resize_image                                              PRIVATE 
//*******************************************************************
void QBtViewer::resize_image( const bool idle )
{
   if( idle ) QBtShared::disable_updates( this );
   QImage img = image_;
   if( ok_ && scaled_ ) {
      const int w = panel_->width() - 8;
      const int h = panel_->height() - 8;
      if( ( img.width() > w ) || ( img.height() > h ) ) {
         img = img.scaled( QSize( w, h ), Qt::KeepAspectRatio );
      }
   }
   picture_->display( img );
   if( idle ) {
      QBtShared::enable_updates( this );
      update();
   }
}
// end of resize_image

//*******************************************************************
// scale_changed                                        PRIVATE slot
//*******************************************************************
void QBtViewer::scale_changed( const int state )
{
   scaled_ = ( Qt::Checked == state );
   resize_image();
}
// end of scale_changed

//*******************************************************************
// next                                                 PRIVATE slot
//*******************************************************************
void QBtViewer::next()
{
   const unsigned int end_idx = files_.size() - 1;

   if( index_ < end_idx ) {
      ++index_;
      display();
   }
}
// end of next

//*******************************************************************
// prev                                                 PRIVATE slot
//*******************************************************************
void QBtViewer::prev()
{
   if( index_ > 0 ) {
      --index_;
      display();
   }
}
// end of prev

//*******************************************************************
// left                                                 PRIVATE slot
//*******************************************************************
void QBtViewer::left()
{
   QBtShared::disable_updates( this );
   QMatrix matrix;
   matrix.rotate( -90 );
   angle_ -= 90;
   if( -360 == angle_ ) angle_ = 0;
   image_ = image_.transformed( matrix );
   resize_image( false );
   check_buttons();
   QBtShared::enable_updates( this );
   update();
}
// end of left

//*******************************************************************
// right                                                PRIVATE slot
//*******************************************************************
void QBtViewer::right()
{
   QBtShared::disable_updates( this );
   QMatrix matrix;
   matrix.rotate( 90 );
   angle_ += 90;
   if( 360 == angle_ ) angle_ = 0;
   image_ = image_.transformed( matrix );
   resize_image( false );
   check_buttons();
   QBtShared::enable_updates( this );
   update();
}
// end of right

//*******************************************************************
// display                                                   PRIVATE
//*******************************************************************
void QBtViewer::display()
{
   if( files_.empty() ) {
      check_buttons();
      image_ = QImage();
      resize_image();
      update();
      return;
   }
   
   QBtShared::disable_updates( this );
   QBtMagic magic( this );
   const QString fpath = files_[ index_ ];
   ok_ = magic.read( fpath );
   if( ok_ ) angle_ = 0;
   else QMessageBox::critical( this, tr( Caption ), tr( Error ).arg( fpath ) );
   image_ = magic.qimage();
   resize_image( false );
   dynamic_cast<QBtView*>( parent() )->current_file( fpath );
   check_buttons();
   QBtShared::enable_updates( this );
   update();
}
// end of display

//*******************************************************************
// check_buttons                                             PRIVATE
//*******************************************************************
void QBtViewer::check_buttons()
{
   if( files_.empty() ) {
      scale_->setEnabled( false );
      left_->setEnabled( false );
      right_->setEnabled( false );
      prev_->setEnabled( false );
      next_->setEnabled( false );
      save_->setEnabled( false );
      delete_->setEnabled( false );
      check_->setEnabled( false );
      setWindowTitle( tr( Caption ) );
   }
   else {
      const QString fpath = files_[ index_ ];
      
      check_->setChecked( checked_.find( fpath ) != checked_.end() );
      save_->setEnabled( angle_ != 0 );
      prev_->setEnabled( index_ > 0 );
      next_->setEnabled( index_ < ( files_.size() - 1 ) );
      setWindowTitle( tr( Caption ) + ": " + fpath );
   }
}
// end of check_buttons

//*******************************************************************
// save_clicked                                         PRIVATE slot
//*******************************************************************
void QBtViewer::save_clicked()
{
   QBtShared::disable_updates( this );
   QBtMagic magic( this );
   ok_ = magic.read( files_[ index_ ] );
   if( ok_ ) {
      magic.rotate( angle_ );
      magic.write();
      angle_ = 0;
   }
   image_ = magic.qimage();
   resize_image( false );
   check_buttons();
   QBtShared::enable_updates( this );
   update();
}
// end of save_clicked

//*******************************************************************
// check_clicked                                        PRIVATE slot
//*******************************************************************
void QBtViewer::check_clicked()
{
   const QString fpath = files_[ index_ ];
   
   if( check_->isChecked() ) {
      checked_.insert( fpath );
      dynamic_cast<QBtView*>( parent() )->check_file( fpath );
   }
   else {
      StringSet::iterator it = checked_.find( fpath );
      if( it != checked_.end() ) {
         checked_.erase( it );
         dynamic_cast<QBtView*>( parent() )->check_file( fpath );
      }
   }
}
// end of check_clicked

//*******************************************************************
// delete_clicked                                       PRIVATE slot
//*******************************************************************
void QBtViewer::delete_clicked()
{
   const QString fpath = files_[ index_ ];
   
   if( QFile::remove( fpath ) ) {
      dynamic_cast<QBtView*>( parent() )->delete_file( fpath );
      files_.erase( files_.begin() + index_ );
      if( index_ >= files_.size() ) {
         index_ = files_.size() - 1;
      } 
      display();
   }
   else {
      QMessageBox::warning( this, tr( Caption ), tr( DeleteError ).arg( fpath ) );
   }
}
// end of delete_clicked
