/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtWorkspace.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 8.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtWorkspace.h"
#include "QBtEventsController.h"
#include "QBtViewWidget.h"
#include "QBtCmdWidget.h"
#include "QBtOutputWidget.h"
#include "QBtView.h"
#include "QBtMagic.h"
#include "QBtShared.h"
#include "QBtProcessDialog.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTabBar>
#include <QStackedLayout>
#include <QLabel>
#include <QFrame>
#include <QTextEdit>
#include <QMessageBox>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtWorkspace::Dir     = QT_TR_NOOP( "Current directory: " );
const char* const QBtWorkspace::Files   = QT_TR_NOOP( "Files" );
const char* const QBtWorkspace::Cmd     = QT_TR_NOOP( "Commands" );
const char* const QBtWorkspace::Output  = QT_TR_NOOP( "Output" );
const char* const QBtWorkspace::Process = QT_TR_NOOP( "Processing" );
const char* const QBtWorkspace::NoCmd   = QT_TR_NOOP( "Nothing to do.\nCommands editor is empty!" );
const char* const QBtWorkspace::SameDirs= QT_TR_NOOP( "Both directories are the same.\nIt is illegal." );

//*******************************************************************
// QBtWorkspace                                          CONSTRUCTOR
//*******************************************************************
QBtWorkspace::QBtWorkspace( QWidget* const in_parent )
: QWidget      ( in_parent )
, bar_         ( new QTabBar )
, stack_       ( new QStackedLayout )
, path_        ( new QLabel( QString() ) )
, view_widget_ ( new QBtViewWidget   )
, cmd_widget_  ( new QBtCmdWidget    )
, out_widget_  ( new QBtOutputWidget )
{
   path_->setAlignment( Qt::AlignLeft );
   /*
   QPalette p = path_->palette();
   p.setColor( path_->foregroundRole(), Qt::darkBlue );
   path_->setPalette( p );
   */

   QHBoxLayout* const path_layout = new QHBoxLayout;
   path_layout->addWidget( new QLabel( tr( Dir ) ) );
   path_layout->addWidget( path_ );
   path_layout->addStretch();
   path_layout->setContentsMargins( 4, 2, 4, 2 );
   QFrame* const path_panel = new QFrame;
   path_panel->setFrameShape( QFrame::Panel );
   path_panel->setFrameShadow( QFrame::Raised );
   path_panel->setLayout( path_layout );
   
   bar_->insertTab( FilesIndex, tr( Files ) );
   bar_->insertTab( CmdIndex, tr( Cmd ) );
   bar_->insertTab( OutputIndex, tr( Output ) );
   bar_->setFocusPolicy( Qt::NoFocus );
   QHBoxLayout* const bar_layout = new QHBoxLayout;
   bar_layout->addWidget( bar_ );
   bar_layout->addStretch();
   bar_layout->setContentsMargins( 4, 2, 4, 0 );

   stack_->insertWidget( FilesIndex , view_widget_ );
   stack_->insertWidget( CmdIndex   , cmd_widget_  );
   stack_->insertWidget( OutputIndex, out_widget_  );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->setSpacing( 0 );
   main_layout->setContentsMargins( 0, 0, 0, 4 );
   main_layout->addWidget( path_panel );
   main_layout->addLayout( bar_layout );
   main_layout->addLayout( stack_ );
   setLayout( main_layout );

   setFocusPolicy( Qt::NoFocus );

   connect( bar_, SIGNAL( currentChanged(int) ), this, SLOT( tab_changed(int) ) );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::DirChanged );
   evc->append( this, QBtEvent::F5 );
   evc->append( this, QBtEvent::SwitchToMessages );
}
// end of QBtWorkspace

//*******************************************************************
// ~QBtWorkspace                                          DESTRUCTOR
//*******************************************************************
QBtWorkspace::~QBtWorkspace()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtWorkspace

//*******************************************************************
// tab_changed                                          PRIVATE slot
//*******************************************************************
void QBtWorkspace::tab_changed( const int in_idx )
{
   stack_->setCurrentIndex( in_idx );
}
// end of tab_changed

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtWorkspace::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::DirChanged:
         path_->setText( event->data( 0 ).toString() );
         break;
      case QBtEvent::F5:
         process();
         break;
      case QBtEvent::SwitchToMessages:
         bar_->setCurrentIndex( OutputIndex );
         break;
   }
}
// end of customEvent

//*******************************************************************
// process                                                   PRIVATE
//*******************************************************************
void QBtWorkspace::process()
{
   QBtView* const viewA = view_widget_->viewA();
   QBtView* const viewB = view_widget_->viewB();
   if( !viewA || !viewB ) return;

   QBtView* src = 0;
   QBtView* dst = 0;
   if( viewA->hasFocus() ) {
      src = viewA;
      dst = viewB;
   }
   else if( viewB->hasFocus() ) {
      src = viewB;
      dst = viewA;
   }
   else {
      return;
   }

   if( src->path() == dst->path() ) {
      QMessageBox::information( this, tr( Process ), tr( SameDirs ) );
      return;
   }

   const QStringList commands = cmd_widget_->commands();
//   const int cmd_count = commands.size();
   if( commands.empty() ) {
      QMessageBox::information( this, tr( Process ), tr( NoCmd ) );
      return;
   }
   
   Strings files = src->get_selected();
   if( files.empty() ) {
      const QString current_fpath = src->get_current();
      if( current_fpath.isEmpty() ) return;
      files.push_back( current_fpath );
   }

   QBtProcessDialog dialog( this, files, dst->path(), commands );
   if( QDialog::Rejected == dialog.exec() ) {
      QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
   }
   dst->reread();
   /*
   // PETLA ROBOCZA
   bool is_ok = true;
   QBtMagic magic( this );
   Strings::const_iterator it = files.begin();
   const Strings::const_iterator end = files.end();

   QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
   QBtShared::disable_updates( this );   
   while( it != end ) {
      magic.read( *it );
      //...................................................
      for( int i = 0; i < cmd_count; ++i ) {
         if( !magic.process( commands[i] ) ) {
            if( is_ok ) is_ok = false;
         }
      }
      //...................................................
      magic.write( dst->path() );
      ++it;
   }
   QBtShared::enable_updates( this );
   dst->reread();

   if( !is_ok ) {
      QBtEventsController::instance()->send_event( QBtEvent::SwitchToMessages );
   }
   */
}
// end of process
